#include "Board.h"

Board::Board(unsigned int size)
    :m_size(size)
{
    if(size=0) m_table=0;
    else {
        m_table=new char[m_size*m_size];
        for(int i=0;i<m_size*m_size;++i) m_table[i]=EMPTY;
    }
}

Board::Board(const Board& board)
    :m_size(board.Size())
{
    int i,j;
    m_table=new char[m_size*m_size];
    for(i=0;i<m_size;++i)
        for(j=0;j<m_size;++j)
            m_table[i*m_size+j]=board(i,j);
    m_black_captures+=board.GetBlackCaptures();
    m_white_captures+=board.GetWhiteCaptures();
}

Board& Board::operator=(const Board& board)
{
    if(&board==this) return *this;
    else {
        m_black_captures+=board.GetBlackCaptures();
        m_white_captures+=board.GetWhiteCaptures();
        if(!m_table){
            m_size=board.Size();
            if(board.IsAllocated()){
                m_table=new char[m_size*m_size];
                int i,j;
                for(i=0;i<m_size;++i)
                    for(j=0;j<m_size;++j)
                        m_table[i*m_size+j]=board(i,j);
                return *this;
            }
        } else {
            if(board.Size()!=m_size)
                throw BoardError("Board::operator=: Invalid assignement,"
                             "boards not of the same size.");
            if(board.IsAllocated()){
                int i,j;
                for(i=0;i<m_size;++i)
                    for(j=0;j<m_size;++j)
                        m_table[i*m_size+j]=board(i,j);
                return *this;
            } else {
                throw BoardError("Board::operator=: Invalid assignement,"
                                 "the board to assign from has no allocated board state"
                                 "but the one to assign to has.");
            }
        }
    }
}

void Board::AllocateBoard()
{
    if(m_table)
        throw BoardError("Board::AllocateBoard: cannot allocate, board already allocated");
    else if(!m_size)
        throw BoardError("Board::AllocateBoard: The board size is not defined");
    else{
        m_table=new char[m_size*m_size];
        int i,j;
        for(i=0;i<m_size;++i)
            for(j=0;j<m_size;++j)
                m_table[i*m_size+j]=EMPTY;
    }
}

void Board::FreeBoard()
{
    if(!m_table)
        throw BoardError("Board::FreeBoard: cannot free, board not allocated");
    else{
        m_size=0;
        delete [] m_table;
        m_table=0;
    }
}

char& Board::operator()(const int& x, const int& y)
{
    if(x<0 || x>=m_size || y<0 || y>= m_size)
        throw BoardError("Board::operator()(const int& x, const int& y): coordinates are out of bounds.");
    return m_table[x*m_size+y];
}

char& Board::operator()(const Coord& c)
{
    if(c.i()<0 || c.i()>=m_size || c.j()<0 || c.j()>= m_size)
        throw BoardError("Board::operator()(const int& x, const int& y): coordinates are out of bounds.");
    return m_table[c.i()*m_size+c.j()];
}

void Board::SetSize(unsigned int size)
{
    if(m_table) throw BoardError("Board::SetSize: The board state is already allocated, the size cannot be changed.");
    else m_size=size;
}

ostream& operator<<(ostream& os, const Board& board)
{
   if(!board.Size()) return os;
   else {
       os<<"  a";
       for(int i=1;i<board.Size();++i)
           os<<"   "<<(char)(((int)'a')+i);
       os<<endl;
       for(int i=0;i<board.Size()-1;++i){
           os<<(char)(((int)'a')+i)<<" ";
           if(board(0,i)==EMPTY) os<<'+';
           else if(board(0,i)==BLACK) os<<'X';
           else os<<'O';
           for(int j=1;j<board.Size();++j){
               if(board(j,i)==EMPTY) os<<"---+";
               else if(board(j,i)==BLACK) os<<"---X";
               else os<<"---O";
           }
           os<<" "<<(char)(((int)'a')+i)<<endl<<"  |";
           for(int j=1;j<board.Size();++j)
               os<<"   |";
           os<<endl;
       }
       os<<(char)(((int)'a')+board.Size()-1)<<" ";
       if(board(0,board.Size()-1)==EMPTY) os<<'+';
       else if(board(0,board.Size()-1)==BLACK) os<<'X';
       else os<<'O';
       for(int j=1;j<board.Size();++j){
           if(board(j,board.Size()-1)==EMPTY) os<<"---+";
           else if(board(j,board.Size()-1)==BLACK) os<<"---X";
           else os<<"---O";
       }
       os<<" "<<(char)(((int)'a')+board.Size()-1)<<endl;
       os<<"  a";
       for(int i=1;i<board.Size();++i)
           os<<"   "<<(char)(((int)'a')+i);
       os<<endl;
       return os;
   }
}

ostream& operator<<(ostream& out,const BoardError& err)
{
    out<<err.what()<<endl;
    return out;
}
