#include "GamePropertiesDialog.h"

GamePropertiesDialog::GamePropertiesDialog(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& refBuilder)
    :Gtk::Dialog(cobject), m_refBuilder(refBuilder), m_node(0), m_computer(false)
{
    Gtk::Button *button_ok=0,*button_close=0;
    m_refBuilder->get_widget("gamepropbutton_ok",button_ok);
    button_ok->signal_clicked().connect(sigc::mem_fun(this,&GamePropertiesDialog::on_ok));
    m_refBuilder->get_widget("gamepropbutton_close",button_close);
    button_close->signal_clicked().connect(sigc::mem_fun(this,&GamePropertiesDialog::on_close));
    Gtk::Table* table;
    m_refBuilder->get_widget("table_gameprops",table);
    table->attach(m_combo_rule,3,4,0,1);
    m_combo_rule.append_text(_("Japanese"));
    m_combo_rule.append_text(_("American Go Association"));
    m_combo_rule.append_text(_("Ing rules of Go"));
    m_combo_rule.append_text(_("New Zealand rules"));
    m_combo_rule.append_text("");
    m_combo_rule.set_active_text("Japanese");
    table->attach(m_combo_black,1,2,1,2);
    m_combo_black.append_text(_("Black player"));
    m_combo_black.signal_changed().connect(sigc::mem_fun(this,&GamePropertiesDialog::on_black_changed));
    table->attach(m_combo_white,3,4,1,2);
    m_combo_white.append_text(_("White player"));
    m_combo_white.signal_changed().connect(sigc::mem_fun(this,&GamePropertiesDialog::on_white_changed));
    table->show_all_children();
}

void GamePropertiesDialog::AttachSgfNode(SgfNode* node)
{
    m_node=node;
    if(m_node->Property(SgfProperty::SZ)){
        Glib::RefPtr<Gtk::SpinButton>::cast_dynamic(m_refBuilder->get_object("spinbutton_size"))->set_sensitive(false);
        try{
        Glib::RefPtr<Gtk::Adjustment>::cast_dynamic(m_refBuilder->get_object("adjustment_size"))
            ->set_value(m_node->Property(SgfProperty::SZ)->Value2I());
        } catch (const SgfPropertyError& err){
            Logger::log()<<"GamePropertiesDialog::AttachSgfNode: "<<err.what()<<endl;
            Glib::RefPtr<Gtk::Adjustment>::cast_dynamic(m_refBuilder->get_object("adjustment_size"))
            ->set_value(19);
        }
    } else {
        Glib::RefPtr<Gtk::Adjustment>::cast_dynamic(m_refBuilder->get_object("adjustment_size"))
            ->set_value(19);
    }
    if(m_node->Property(SgfProperty::PB)){
        try{
            ustring pl_black=m_node->Property(SgfProperty::PB)->Value2U();
            if(pl_black!=_("Black player")){
                m_combo_black.append_text(pl_black);
                m_combo_black.set_active_text(pl_black);
                m_combo_black.remove_text(_("Black player"));
            } else m_combo_black.set_active_text(_("Black player"));
        } catch (const SgfPropertyError& err){
            Logger::log()<<"GamePropertiesDialog::AttachSgfNode: "<<err.what()<<endl;
            m_combo_black.set_active_text(_("Black player"));
        }
    } else {
        m_combo_black.set_active_text(_("Black player"));
    }
    if(m_node->Property(SgfProperty::PW)){
        try{
            ustring pl_white=m_node->Property(SgfProperty::PW)->Value2U();
            if(pl_white!=_("White player")){
                m_combo_white.append_text(pl_white);
                m_combo_white.set_active_text(pl_white);
                m_combo_white.remove_text(_("White player"));
            } else m_combo_white.set_active_text(_("White player"));
        } catch (const SgfPropertyError& err){
            Logger::log()<<"GamePropertiesDialog::AttachSgfNode: "<<err.what()<<endl;
            m_combo_white.set_active_text(_("White player"));
        }
    } else {
        m_combo_white.set_active_text(_("White player"));
    }
    if(m_node->Property(SgfProperty::PC)){
        try{
        Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("entry_place"))
            ->set_text(m_node->Property(SgfProperty::PC)->Value2U());
        } catch(const SgfPropertyError& err) {
            Logger::log()<<err.what()<<endl;
            Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("entry_place"))
            ->set_text("");
        }
    } else {
        Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("entry_place"))
            ->set_text("");
    }
    if(m_node->Property(SgfProperty::DT)){
        try{
        Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("entry_date"))
            ->set_text(m_node->Property(SgfProperty::DT)->Value2U());
        } catch (const SgfPropertyError& err){
            Logger::log()<<"GamePropertiesDialog::AttachSgfNode: "<<err.what()<<endl;
            Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("adjustment_size"))
            ->set_text("");
        }
    } else {
        Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("entry_date"))
            ->set_text("");
    }
    if(m_node->Property(SgfProperty::C)){
        try{
        Glib::RefPtr<Gtk::TextBuffer>::cast_dynamic(m_refBuilder->get_object("textbuffer_properties_comment"))
            ->set_text(m_node->Property(SgfProperty::C)->Value2U());
        } catch (const SgfPropertyError& err){
            Logger::log()<<"GamePropertiesDialog::AttachSgfNode: "<<err.what()<<endl;
            Glib::RefPtr<Gtk::Entry>::cast_dynamic(m_refBuilder->get_object("adjustment_size"))
            ->set_text("");
        }
    } else {
        Glib::RefPtr<Gtk::TextBuffer>::cast_dynamic(m_refBuilder->get_object("textbuffer_properties_comment"))
            ->set_text("");
    }
    if(m_node->Property(SgfProperty::RU)){
        try{
            ustring rule=m_node->Property(SgfProperty::RU)->Value2U();
            if(rule=="Japanese") m_combo_rule.set_active_text("Japanese");
            else if(rule=="AGA") m_combo_rule.set_active_text("American Go Association");
            else if(rule=="GOE") m_combo_rule.set_active_text("Ing rules of Go");
            else if(rule=="NZ") m_combo_rule.set_active_text("New Zealand rules");
            else m_combo_rule.prepend_text(m_node->Property(SgfProperty::RU)->Value2U());
        } catch (const SgfPropertyError& err){
            Logger::log()<<"GamePropertiesDialog::AttachSgfNode: "<<err.what()<<endl;
            m_combo_rule.set_active_text("");
        }
    } else {
        m_combo_rule.set_active_text("Japanese");
    }
}

void GamePropertiesDialog::SetComputerGame(bool comp)
{
    m_computer=comp;
    if(m_computer){
        for(int level=0; level<11;++level){
            m_combo_black.append_text(ustring::compose(_("Gnugo level %1"),level));
            m_combo_white.append_text(ustring::compose(_("Gnugo level %1"),level));
        }
        m_combo_white.set_active_text(ustring::compose(_("Gnugo level %1"),10));
    }
}

void GamePropertiesDialog::on_black_changed()
{
}
void GamePropertiesDialog::on_white_changed()
{
}

void GamePropertiesDialog::on_ok()
{
    if(m_node){
        if(!m_node->Property(SgfProperty::SZ)){
            Gtk::SpinButton* size=0;
            m_refBuilder->get_widget("spinbutton_size",size);
            m_node->AddProperty(SgfProperty::SZ, ustring::compose("%1",size->get_value_as_int()));
        }

        if(m_node->Property(SgfProperty::PB)){
            *(m_node->Property(SgfProperty::PB)->Begin())=m_combo_black.get_entry()->get_text();
        } else if(!m_combo_black.get_entry()->get_text().empty())
            m_node->AddProperty(SgfProperty::PB,m_combo_black.get_entry()->get_text());

        if(m_node->Property(SgfProperty::PW)){
            *(m_node->Property(SgfProperty::PW)->Begin())=m_combo_white.get_entry()->get_text();
        } else if(!m_combo_white.get_entry()->get_text().empty())
            m_node->AddProperty(SgfProperty::PW,m_combo_white.get_entry()->get_text());

        Gtk::Entry* place=0;
        m_refBuilder->get_widget("entry_place",place);
        if(m_node->Property(SgfProperty::PC)){
            if(place->get_text().empty())
                m_node->RemoveProperty(m_node->Property(SgfProperty::PC));
            else
                *(m_node->Property(SgfProperty::PC)->Begin())=place->get_text();
        } else if(!place->get_text().empty())
            m_node->AddProperty(SgfProperty::PC,place->get_text());

        Gtk::Entry* date=0;
        m_refBuilder->get_widget("entry_date",date);
        if(m_node->Property(SgfProperty::DT)){
            if(date->get_text().empty())
                m_node->RemoveProperty(m_node->Property(SgfProperty::DT));
            else
                *(m_node->Property(SgfProperty::DT)->Begin())=date->get_text();
        } else if(!date->get_text().empty())
            m_node->AddProperty(SgfProperty::DT,date->get_text());

        Glib::RefPtr<Gtk::TextBuffer> comment=Glib::RefPtr<Gtk::TextBuffer>::cast_dynamic(m_refBuilder->get_object("textbuffer_properties_comment"));
        if(m_node->Property(SgfProperty::C)){
            if(comment->get_text().empty())
                m_node->RemoveProperty(m_node->Property(SgfProperty::C));
            else
                *(m_node->Property(SgfProperty::C)->Begin())=comment->get_text();
        } else if(!comment->get_text().empty())
            m_node->AddProperty(SgfProperty::C,comment->get_text());
        if(m_node->Property(SgfProperty::RU)){
            if(m_combo_rule.get_entry()->get_text().empty())
                m_node->RemoveProperty(m_node->Property(SgfProperty::RU));
            else{
                ustring rule=m_combo_rule.get_entry()->get_text();
                if(rule=="American Go Association") *(m_node->Property(SgfProperty::RU)->Begin())="AGA";
                else if(rule=="Ing rules of Go") *(m_node->Property(SgfProperty::RU)->Begin())="GOE";
                else if(rule=="New Zealand rules") *(m_node->Property(SgfProperty::RU)->Begin())="NZ";
                else *(m_node->Property(SgfProperty::RU)->Begin())=rule;
            }
        } else if(!m_combo_rule.get_entry()->get_text().empty()){
            ustring rule=m_combo_rule.get_entry()->get_text();
            if(rule=="American Go Association") m_node->AddProperty(SgfProperty::RU,"AGA");
            else if(rule=="Ing rules of Go") m_node->AddProperty(SgfProperty::RU,"GOE");
            else if(rule=="New Zealand rules") m_node->AddProperty(SgfProperty::RU,"NZ");
            else m_node->AddProperty(SgfProperty::RU,rule);
        }
    }
    response(Gtk::RESPONSE_OK);
}
