#include "GobanLogic.h"
#include <glibmm/i18n.h>

void GobanLogic::get_group(const Coord& c, list<Coord>& group, list<Coord>& lib)
{
    group.push_back(c);
    const Board& board=m_sgf_current->GetBoard();
    if(c.i()>0){
        if(board(c.i()-1,c.j())==board(c.i(),c.j())){
            if(find(group.begin(),group.end(),Coord(c.i()-1,c.j()))==group.end())
                get_group(Coord(c.i()-1,c.j()),group,lib);
        } else if(board(c.i()-1,c.j())==EMPTY){
            if(find(lib.begin(),lib.end(),Coord(c.i()-1,c.j()))==lib.end())
                lib.push_back(Coord(c.i()-1,c.j()));
        }
    }
    if(c.i()<board.Size()-1){
        if(board(c.i()+1,c.j())==board(c.i(),c.j())){
            if(find(group.begin(),group.end(),Coord(c.i()+1,c.j()))==group.end())
                get_group(Coord(c.i()+1,c.j()),group,lib);
        } else if(board(c.i()+1,c.j())==EMPTY){
            if(find(lib.begin(),lib.end(),Coord(c.i()+1,c.j()))==lib.end())
                lib.push_back(Coord(c.i()+1,c.j()));
        }
    }
    if(c.j()>0){
        if(board(c.i(),c.j()-1)==board(c.i(),c.j())){
            if(find(group.begin(),group.end(),Coord(c.i(),c.j()-1))==group.end())
                get_group(Coord(c.i(),c.j()-1),group,lib);
        } else if(board(c.i(),c.j()-1)==EMPTY){
            if(find(lib.begin(),lib.end(),Coord(c.i(),c.j()-1))==lib.end())
                lib.push_back(Coord(c.i(),c.j()-1));
        }
    }
    if(c.j()<board.Size()-1){
        if(board(c.i(),c.j()+1)==board(c.i(),c.j())){
            if(find(group.begin(),group.end(),Coord(c.i(),c.j()+1))==group.end())
                get_group(Coord(c.i(),c.j()+1),group,lib);
        } else if(board(c.i(),c.j()+1)==EMPTY){
            if(find(lib.begin(),lib.end(),Coord(c.i(),c.j()+1))==lib.end())
                lib.push_back(Coord(c.i(),c.j()+1));
        }
    }
}

GobanLogic::GobanLogic(const ustring& sgfstream)
    :m_black(true), m_modified(false), m_last_passed(false), m_komi(5.5)
{
    if(sgfstream.empty()){
        m_sgf_root=new SgfNode();
        m_sgf_current=m_sgf_root;
    } else {
        m_sgf_root=SgfNode::BuildTree(sgfstream);
        m_sgf_current=m_sgf_root;
    }
}

void GobanLogic::PlayGameTree(SgfNode* node, list<pair<SgfNode*,ustring> > & errors)
{
    if(node==0) node=m_sgf_root;
    try{
        PlayNode(node);
    }catch(const GobanLogicError & err){
        errors.push_back(pair<SgfNode*,ustring>(node,err.what()));
    }
    for(int i=0; i<node->NChilds();++i)
    {
        PlayGameTree(node->ChildAt(i),errors);
    }
}

void GobanLogic::FreeGameTree(SgfNode* node)
{
    node->GetBoard().FreeBoard();
    for(int i=0;i<node->NChilds();++i)
    {
        FreeGameTree(node->ChildAt(i));
    }
}

GobanLogic::~GobanLogic()
{
    delete m_sgf_root;
}

const Board* GobanLogic::SetFinalState(vector<Coord> dead, vector<Coord> black_terr, vector<Coord> white_terr)
{
    SgfNode* final=new SgfNode(m_sgf_current);
    SgfProperty *tb=new SgfProperty(SgfProperty::TB);
    SgfProperty *tw=new SgfProperty(SgfProperty::TW);
    double score_white=m_sgf_current->GetBoard().GetWhiteCaptures()+m_komi, score_black=m_sgf_current->GetBoard().GetWhiteCaptures();
    for(int i=0;i<dead.size();++i){
        if(m_sgf_current->GetBoard()(dead[i])==BLACK){
            score_white+=2;
            final->AddWhiteCaptures(1);
            tw->AddValue((ustring)dead[i]);
        } else {
            score_black+=2;
            final->AddBlackCaptures(1);
            tb->AddValue((ustring)dead[i]);
        }
    }
    for(int i=0;i<black_terr.size();++i){
        score_black+=1;
        tb->AddValue((ustring)black_terr[i]);
    }
    for(int i=0;i<white_terr.size();++i){
        score_white+=1;
        tw->AddValue((ustring)white_terr[i]);
    }
    score_black+=m_sgf_current->GetBoard().GetBlackCaptures();
    score_white+=m_sgf_current->GetBoard().GetWhiteCaptures();
    final->AddProperty(tb);
    final->AddProperty(tw);
    final->SetFinal(true);
    if(score_white==score_black) m_sgf_root->AddProperty(SgfProperty::RE,ustring("Draw"));
    else if(score_white<score_black) m_sgf_root->AddProperty(SgfProperty::RE,ustring::compose("B+%1",score_black-score_white));
    else m_sgf_root->AddProperty(SgfProperty::RE,ustring::compose("W+%1",score_white-score_black));
    return PlayNode(final);
}

const Board* GobanLogic::AddStone(const Coord& coord,bool black)
{
    if(m_sgf_current->Property(SgfProperty::AE)){
        if(m_sgf_current->Property(SgfProperty::AE)->HasValue((ustring)coord))
            m_sgf_current->Property(SgfProperty::AE)->RemoveValue((ustring)coord);
    }
    if(black){
        m_sgf_current->GetBoard()(coord)==BLACK;
        if(m_sgf_current->Property(SgfProperty::AW)){
            try{
                m_sgf_current->Property(SgfProperty::AW)->RemoveValue((ustring)coord);
            }catch(SgfPropertyError & err){}
        }
        if(m_sgf_current->Property(SgfProperty::AB))
            m_sgf_current->Property(SgfProperty::AB)->AddValue((ustring)coord);
        else
            m_sgf_current->AddProperty(SgfProperty::AB,(ustring)coord);
    } else{
        m_sgf_current->GetBoard()(coord)==WHITE;
        if(m_sgf_current->Property(SgfProperty::AB)){
            try{
                m_sgf_current->Property(SgfProperty::AB)->RemoveValue((ustring)coord);
            }catch(SgfPropertyError & err){}
        }
        if(m_sgf_current->Property(SgfProperty::AW))
            m_sgf_current->Property(SgfProperty::AW)->AddValue((ustring)coord);
        else
            m_sgf_current->AddProperty(SgfProperty::AW,(ustring)coord);
    }
    FreeGameTree(m_sgf_current);
    list<pair<SgfNode*,ustring> > errs;
    SgfNode* current=m_sgf_current;
    PlayGameTree(m_sgf_current,errs);
    return PlayNode(current);
}

const Board* GobanLogic::RemoveStone(const Coord& coord)
{
    m_sgf_current->GetBoard()(coord)==EMPTY;
    bool create_prop=true;
    if(m_sgf_current->Property(SgfProperty::AB)){
        if(m_sgf_current->Property(SgfProperty::AB)->HasValue((ustring)coord)){
            m_sgf_current->Property(SgfProperty::AB)->RemoveValue((ustring)coord);
            create_prop=false;
        }
    }
    if(m_sgf_current->Property(SgfProperty::AW)){
        if(m_sgf_current->Property(SgfProperty::AW)->HasValue((ustring)coord)){
            m_sgf_current->Property(SgfProperty::AW)->RemoveValue((ustring)coord);
            create_prop=false;
        }
    }
    if(create_prop){
        if(m_sgf_current->Property(SgfProperty::AE))
            m_sgf_current->Property(SgfProperty::AE)->AddValue((ustring)coord);
        else
            m_sgf_current->AddProperty(SgfProperty::AE,(ustring)coord);
    }
    FreeGameTree(m_sgf_current);
    list<pair<SgfNode*,ustring> > errs;
    SgfNode* current=m_sgf_current;
    PlayGameTree(m_sgf_current,errs);
    return PlayNode(current);
}

void GobanLogic::Resign()
{
    char winner='B';
    if(m_black)
        winner='W';
    m_sgf_root->AddProperty(SgfProperty::RE,ustring::compose("%1+Resign",winner));
    m_sgf_current->SetFinal(true);
}

ustring GobanLogic::GetScore()
{
    SgfNode* beg=m_sgf_current;
    while(beg!=m_sgf_root){
        SgfNode* par=beg->Parent();
        if(par->ChildAt(0)==beg)
            beg=par;
        else{
            beg=par;
            break;
        }
    }
    if(beg->Property(SgfProperty::RE)){
        return beg->Property(SgfProperty::RE)->Value2U();
    } else {
        return ustring("?");
    }
}

const Board* GobanLogic::PlayNode(SgfNode* node)
{
    if(node->GetBoard().Size()){
        m_sgf_current=node;
        if(node->Property(SgfProperty::B)) m_black=false;
        else if(node->Property(SgfProperty::W)) m_black=true;
        else if(node->NChilds() && node->ChildAt(0)->Property(SgfProperty::W)) m_black=false;
        else m_black=true;
        return &node->GetBoard();
    } else {
        if(node==m_sgf_root){
            if(!node->Property(SgfProperty::SZ))
                node->AddProperty(SgfProperty::SZ,"19");
            int size=node->Property(SgfProperty::SZ)->Value2I();
            if(size<=0)
                throw GobanLogicError("GobanLogic::PlayNode(SgfNode*) :\
                                       Root node must have positive size property");
            m_sgf_root->GetBoard().SetSize(size);
            m_sgf_root->GetBoard().AllocateBoard();
            m_sgf_current=m_sgf_root;
            treat_node(node);
            if(!node->Property(SgfProperty::W) && !node->Property(SgfProperty::B)){
                if(node->NChilds() && node->ChildAt(0)->Property(SgfProperty::W)) m_black=false;
                else m_black=true;
            }
            return &node->GetBoard();
        } else {
            node->GetBoard()=*PlayNode(node->Parent());
            m_sgf_current=node;
            treat_node(node);
            if(!node->Property(SgfProperty::W) && ! node->Property(SgfProperty::B)){
                if(node->NChilds() && node->ChildAt(0)->Property(SgfProperty::W)) m_black=false;
                else m_black=true;
            }
            return &node->GetBoard();
        }
    }
}

SgfNode* GobanLogic::NodeBeforeFirstMove()
{
    if(!m_sgf_root->NChilds()) return m_sgf_root;
    SgfNode* node=m_sgf_root, *the_node=0;
    bool found=false;
    while(!found && node){
        if(node->Property(SgfProperty::B) || node->Property(SgfProperty::W))
            found=true;
        else {
            if(node->NChilds()) node=node->ChildAt(0);
            else {
                the_node=node;
                node=0;
            }
        }
    }
    if(found) return node->Parent();
    else return the_node;
}

SgfNode* GobanLogic::PreviousBranchingNode(SgfNode* st)
{
    while(st!=m_sgf_root){
        st=st->Parent();
        if(st->Parent() && st->Parent()->NChilds()>1)
            return st;
    }
    return st;
}

const SgfNode* GobanLogic::PreviousBranchingNode(const SgfNode* st) const
{
    while(st!=m_sgf_root){
        st=st->Parent();
        if(st->Parent() && st->Parent()->NChilds()>1)
            break;
    }
    return st;
}

SgfNode* GobanLogic::NextBranchingNode(SgfNode* st)
{
    while(st->NChilds()){
        if(st->NChilds()>1){
            return st->ChildAt(0);
        }else{
            st=st->ChildAt(0);
        }
    }
    return st;
}

const SgfNode* GobanLogic::NextBranchingNode(const SgfNode* st) const
{
    while(st->NChilds()){
        if(st->NChilds()>1){
            return st->ChildAt(0);
        }else{
            st=st->ChildAt(0);
        }
    }
    return st;
}

SgfNode* GobanLogic::LastNode(SgfNode* node)
{
    while(node->NChilds()){
        node=node->ChildAt(0);
    }
    return node;
}

const SgfNode* GobanLogic::LastNode(const SgfNode* node) const
{
    while(node->NChilds()){
        node=node->ChildAt(0);
    }
    return node;
}

const Board* GobanLogic::GobanLogic::PlayNextNode()
{
    if(m_sgf_current->NChilds()==0)
        throw GobanLogicError("No next move.");
    else
        return PlayNode(m_sgf_current->ChildAt(0));
}

const Board* GobanLogic::PlayPreviousNode()
{
    if(m_sgf_current->Parent()==0)
        throw GobanLogicError("Root node has no parent.");
    else
        return PlayNode(m_sgf_current->Parent());
}

bool GobanLogic::treat_move(Board& board, const Coord& c, bool black)
{
    char col,ncol;
    if(black){
        col=BLACK;
        ncol=WHITE;
    }else{
        col=WHITE;
        ncol=BLACK;
    }
    if(c.i()<0 || c.i()>=board.Size() || c.j()<0 || c.j()>=board.Size()){
        //pass
        m_black=!black;
        if(black) m_sgf_current->AddProperty(SgfProperty::C,_("Black player pass"));
        else m_sgf_current->AddProperty(SgfProperty::C,_("White player pass"));
        if(m_last_passed){
            m_last_passed=false;
            m_sgf_current->SetFinal(true);
        } else {
            m_last_passed=true;
        }
        return true;
    }
    if(board(c)!=EMPTY || c==m_sgf_current->Parent()->GetBoard().GetKo()){
        Logger::log()<<"illegal move at ("<<c<<"). Board state is:"<<board<<endl;
        return false;
    }
    board(c)=col;
    list<Coord> group,groupl,groupr,groupu,groupd, libs;
    if(c.i()>0 && board(c.i()-1,c.j())==ncol){
        get_group(Coord(c.i()-1,c.j()),groupl,libs);
        if(libs.size()==0){
            list<Coord>::iterator iter=groupl.begin();
            while(iter!=groupl.end()){
                board(*iter)=EMPTY;
                iter++;
            }
            if(col==BLACK) board.AddBlackCaptures(groupl.size());
            else board.AddWhiteCaptures(groupl.size());
        } else {
            groupl.clear();
        }
    }
    if(c.i()<m_sgf_current->GetBoard().Size()-1 && board(c.i()+1,c.j())==ncol){
        libs.clear();
        get_group(Coord(c.i()+1,c.j()),groupr,libs);
        if(libs.size()==0){
            list<Coord>::iterator iter=groupr.begin();
            while(iter!=groupr.end()){
                board(*iter)=EMPTY;
                iter++;
            }
            if(col==BLACK) board.AddBlackCaptures(groupr.size());
            else board.AddWhiteCaptures(groupr.size());
        } else {
            groupr.clear();
        }
    }
    if(c.j()>0 && board(c.i(),c.j()-1)==ncol){
        libs.clear();
        get_group(Coord(c.i(),c.j()-1),groupd,libs);
        if(libs.size()==0){
            list<Coord>::iterator iter=groupd.begin();
            while(iter!=groupd.end()){
                board(*iter)=EMPTY;
                iter++;
            }
            if(col==BLACK) board.AddBlackCaptures(groupd.size());
            else board.AddWhiteCaptures(groupd.size());
        } else {
            groupd.clear();
        }
    }
    if(c.j()<m_sgf_current->GetBoard().Size()-1 && board(c.i(),c.j()+1)==ncol){
        libs.clear();
        get_group(Coord(c.i(),c.j()+1),groupu,libs);
        if(libs.size()==0){
            list<Coord>::iterator iter=groupu.begin();
            while(iter!=groupu.end()){
                board(*iter)=EMPTY;
                iter++;
            }
            if(col==BLACK) board.AddBlackCaptures(groupu.size());
            else board.AddWhiteCaptures(groupu.size());
        } else {
            groupu.clear();
        }
    }
    libs.clear();
    get_group(c,group,libs);
    if(libs.size()==0){
        board(c)=EMPTY;
            Logger::log()<<"illegal (no freedoms) move at ("<<c<<"). Board state is:"<<board<<endl;
        return false;
    } else if(libs.size()==1 && group.size()==1 && (groupr.size()+groupl.size()+groupu.size()+groupd.size())==1){
        bool ko=true;
        if(groupl.size()==1){
            Coord nc=*groupl.begin();
            /*groupu.clear();
            groupd.clear();
            groupl.clear();
            if(c.i()>0){
                get_group(Coord(nc.i()-1,nc.j()),groupl,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.j()<m_sgf_current->GetBoard().Size()-1){
                get_group(Coord(nc.i(),nc.j()+1),groupu,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.j()>0){
                get_group(Coord(nc.i(),nc.j()-1),groupd,libs);
                if(libs.size()==1) ko=true;
            }*/
            if(ko) board.SetKo(nc);
            m_black=!black;
            return true;
        } else if(groupr.size()==1){
            Coord nc=*groupr.begin();
            /*groupu.clear();
            groupd.clear();
            groupr.clear();
            if(c.i()>m_sgf_current->GetBoard().Size()-1){
                get_group(Coord(nc.i()+1,nc.j()),groupr,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.j()<m_sgf_current->GetBoard().Size()-1){
                get_group(Coord(nc.i(),nc.j()+1),groupu,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.j()>0){
                get_group(Coord(nc.i(),nc.j()-1),groupd,libs);
                if(libs.size()==1) ko=true;
            }*/
            if(ko) board.SetKo(nc);
            m_black=!black;
            return true;
        } else if(groupd.size()==1){
            Coord nc=*groupd.begin();
            /*groupl.clear();
            groupr.clear();
            groupd.clear();
            if(c.i()>m_sgf_current->GetBoard().Size()-1){
                get_group(Coord(nc.i()+1,nc.j()),groupr,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.i()<0){
                get_group(Coord(nc.i()-1,nc.j()),groupl,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.j()>0){
                get_group(Coord(nc.i(),nc.j()-1),groupd,libs);
                if(libs.size()==1) ko=true;
            }*/
            if(ko) board.SetKo(nc);
            m_black=!black;
            return true;
        } else if(groupu.size()==1){
            Coord nc=*groupu.begin();
            /*groupl.clear();
            groupr.clear();
            groupu.clear();
            if(c.i()>m_sgf_current->GetBoard().Size()-1){
                get_group(Coord(nc.i()+1,nc.j()),groupr,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.i()<0){
                get_group(Coord(nc.i()-1,nc.j()),groupl,libs);
                if(libs.size()==1) ko=true;
            }
            if(c.j()>m_sgf_current->GetBoard().Size()-1){
                get_group(Coord(nc.i(),nc.j()+1),groupu,libs);
                if(libs.size()==1) ko=true;
            }*/
            if(ko) board.SetKo(nc);
            m_black=!black;
            return true;
        } 
    } else {
        m_black=!black;
        board.SetKo(Coord(-1,-1));
        return true;
    }
}

const Board* GobanLogic::PlayMove(const Coord& c)
{
    bool legal,newNode=false;
    SgfNode* next=0;
    //Checks whether the move exists already.
    for(int ci=0;ci<m_sgf_current->NChilds();++ci){
        SgfProperty* move=0;
        if(m_black) move=m_sgf_current->ChildAt(ci)->Property(SgfProperty::B);
        else move=m_sgf_current->ChildAt(ci)->Property(SgfProperty::W);
        if(move && move->Value2U()==(ustring)c){
            next=m_sgf_current->ChildAt(ci);
            return PlayNode(next);
        }
    }
    //Create a new node if it doesn't
    if(next==0){
        next= new SgfNode(m_sgf_current);
        SgfProperty* move;
        if(m_black) move=new SgfProperty(SgfProperty::B,(ustring)c);
        else move=new SgfProperty(SgfProperty::W,(ustring)c);
        next->AddProperty(move);
        newNode=true;
    }
    if(!next->Parent())//impossible in principle. I keep it for debugging.
        throw GobanLogicError("GobanLogic::PlayMove: Root node cannot contain a move");
    else if(next->GetBoard().Size()==0)
        next->GetBoard()=next->Parent()->GetBoard();
    m_sgf_current=next;
    legal=treat_move(next->GetBoard(), c,m_black);
    if(!legal){
        m_sgf_current=m_sgf_current->Parent();
        if(newNode){
            delete next;
            throw GobanLogicError("GobanLogic::PlayMove: Illegal move",false,1);
        } else {
            throw GobanLogicError("GobanLogic::PlayMove: Illegal move in game tree",true);
        }
    }
    m_modified=true;
    return &next->GetBoard();
}

void GobanLogic::treat_node(SgfNode* node)
{
    list<SgfProperty*>::const_iterator iprop=node->Begin();
    while(iprop!=node->End()){
        switch((*iprop)->PropIdent()){
            case SgfProperty::AB:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        try{
                            node->GetBoard()(Coord(*iter))=BLACK;
                        } catch(const BoardError& err) {
                        }
                        iter++;
                    }
                }
                break;
            case SgfProperty::AW:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        try{
                            node->GetBoard()(Coord(*iter))=WHITE;
                        } catch(const BoardError& err) {
                        }
                        iter++;
                    }
                }
                break;
            case SgfProperty::AE:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        try{
                            node->GetBoard()(Coord(*iter))=EMPTY;
                        } catch(const BoardError& err) {
                        }
                        iter++;
                    }
                }
                break;
            case SgfProperty::B:
                {
                    bool legal=treat_move(node->GetBoard(),(*iprop)->Value2C(),true);
                    if(!legal)
                        throw GobanLogicError("GobanLogic::treat_node: Illegal move in game tree",true);
                }
                break;
            case SgfProperty::W:
                {
                    bool legal=treat_move(node->GetBoard(),(*iprop)->Value2C(),false);
                    if(!legal)
                        throw GobanLogicError("GobanLogic::treat_node: Illegal move in game tree",true);
                }
                break;
            case SgfProperty::CR:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        node->GetBoard().AddLabel(Coord(*iter),Label::CIRCLE);
                        iter++;
                    }
                }
                break;
            case SgfProperty::SQ:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        node->GetBoard().AddLabel(Coord(*iter),Label::SQUARE);
                        iter++;
                    }
                }
                break;
            case SgfProperty::TR:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        node->GetBoard().AddLabel(Coord(*iter),Label::TRIANGLE);
                        iter++;
                    }
                }
                break;
            case SgfProperty::LB:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        Coord point;
                        ustring letter;
                        try{
                        point=(*iprop)->Value2C(iter,false);
                        letter=(*iprop)->Value2U(iter,true);
                        } catch (SgfPropertyError err) {
                            Logger::log()<<err.what()<<endl;
                        } catch (CoordError err) {
                            Logger::log()<<err.what()<<endl;
                        }
                        node->GetBoard().AddLabel(point,Label::LETTER,letter[0]);
                        iter++;
                    }
                }
                break;
            case SgfProperty::TB:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        Coord point;
                        try{
                            point=(*iprop)->Value2C(iter,false);
                        }catch (SgfPropertyError err) {
                            Logger::log()<<err.what()<<endl;
                        } catch (CoordError err) {
                            Logger::log()<<err.what()<<endl;
                        }
                        node->GetBoard().AddBlackTerritory(point);
                        iter++;
                    }
                }
                break;
            case SgfProperty::TW:
                {
                    list<ustring>::const_iterator iter=(*iprop)->Begin();
                    while(iter!=(*iprop)->End()){
                        Coord point;
                        try{
                            point=(*iprop)->Value2C(iter,false);
                        }catch (SgfPropertyError err) {
                            Logger::log()<<err.what()<<endl;
                        } catch (CoordError err) {
                            Logger::log()<<err.what()<<endl;
                        }
                        node->GetBoard().AddWhiteTerritory(point);
                        iter++;
                    }
                }
                break;
            default:
                {
                    /* skip */
                }
                break;
        }
        iprop++;
    }
}

ostream& operator<<(ostream& os,const GobanLogicError& err)
{
    if(err.Fatal())
        os<<"Fatal: ";
    os<<"GobanLogicError: "<<err.what();
    return os;
}
