#ifndef _GOBANWIDGET_H
#define _GOBANWIDGET_H

#include <gtkmm.h>
#include <cstring>

#include "Board.h"

//! A derived class of Gtk::Widget to display a goban
class GobanWidget : public Gtk::Widget
{
public:
    //!Constructor using the Gtk::Builder
    GobanWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& refBuilder);
    //!Destructor
    ~GobanWidget();
    //!Set the board to display
    void SetBoard(const Board* board);
    //!Set the color of the cursor.
    void SetBlack(bool black) {if(black) m_cur_col=BLACK; else m_cur_col=WHITE;}
    //!Set the cursor void (no color).
    void SetVoidCursor() {m_cur_col=EMPTY;}
    //!Show the coordinates on the goban.
    void SetShowTicks(bool show) {m_draw_ticks=show; draw_goban_pixmap(); queue_draw();}
    //!signal emitted when the user click on the goban (drops a stone since it is not yet known
    //if the move is legal or not)
    sigc::signal<void,Coord> signal_stone_dropped() {return m_signal_stone_dropped;}
protected:
    virtual bool on_expose_event(GdkEventExpose* event);
    virtual bool on_button_press_event(GdkEventButton* event);
    virtual bool on_motion_notify_event(GdkEventMotion* event);
    virtual bool on_leave_notify_event(GdkEventCrossing* event);
    virtual void on_size_request(Gtk::Requisition* requisition);
    virtual void on_size_allocate(Gtk::Allocation& allocation);
    virtual void on_realize();
    virtual void on_unrealize();
    virtual void on_map();
    virtual void on_unmap();
private:
    //!Load the pixbuffers from the data directory
    void load_pixbuffers();
    void draw_goban_pixmap();
    //!Get the top left corner of the goban, that is the (0,0) intersection.
    void top_left_corner(int& x, int& y);
    //!Draw the goban wood
    void draw_goban_wood(Glib::RefPtr<Gdk::GC> gc, Cairo::RefPtr<Cairo::Context> cr, int x=-1, int y=-1, int w=-1, int h=-1);
    //!Draw the stones
    void draw_stones(Glib::RefPtr<Gdk::GC> gc, Cairo::RefPtr<Cairo::Context> cr, int x=-1, int y=-1, int w=-1, int h=-1);
    //!Draw the labels
    void draw_labels(Glib::RefPtr<Gdk::GC> gc,Cairo::RefPtr<Cairo::Context> cr, bool stone_labels=false);
    void draw_ko(Glib::RefPtr<Gdk::GC> gc,Cairo::RefPtr<Cairo::Context> cr);
    void draw_territory(Glib::RefPtr<Gdk::GC> gc,Cairo::RefPtr<Cairo::Context> cr);
    sigc::signal<void,Coord> m_signal_stone_dropped;
    int m_cur_x;
    int m_cur_y;
    int m_cur_col;
    bool m_draw_ticks;
    int m_line_space;
    //!The board to display. Not owned by the widget.
    const Board* m_board;
    Glib::RefPtr<Gdk::Window> m_refGdkWindow;
    Glib::RefPtr<Gdk::Pixbuf> m_board_pix;
    Glib::RefPtr<Gdk::Pixbuf> m_stone_black;
    Glib::RefPtr<Gdk::Pixbuf> m_stone_white;
    Glib::RefPtr<Gdk::Pixbuf> m_shadow;
    Glib::RefPtr<Gdk::Pixbuf> m_stone_black_cursor;
    Glib::RefPtr<Gdk::Pixbuf> m_stone_white_cursor;
    Glib::RefPtr<Gdk::Pixbuf> m_void_cursor;
    Glib::RefPtr<Gdk::Pixbuf> m_logo;
    Glib::RefPtr<Gdk::Pixmap> m_goban_map;
};

#endif // _GOBANWIDGET_H
