#include <glibmm/i18n.h>
#include "GobanWindow.h"

GobanWindow::GobanWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& refBuilder)
    :Gtk::Window(cobject), m_refBuilder(refBuilder), m_goban_logic(0), m_node_model(), m_opponent(0), m_opponent_to_play(false), m_problem(false)
{
    m_folder=Gio::File::create_for_path(Glib::get_home_dir());
    Gtk::IconTheme::add_builtin_icon("play",64,Gdk::Pixbuf::create_from_file(DATADIR"play.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("add_black",64,Gdk::Pixbuf::create_from_file(DATADIR"add_black.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("add_white",64,Gdk::Pixbuf::create_from_file(DATADIR"add_white.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("pass",64,Gdk::Pixbuf::create_from_file(DATADIR"pass.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("player_black",64,Gdk::Pixbuf::create_from_file(DATADIR"black.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("player_white",64,Gdk::Pixbuf::create_from_file(DATADIR"white.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("next_branch",64,Gdk::Pixbuf::create_from_file(DATADIR"next_branch.svg",64,64));
    Gtk::IconTheme::add_builtin_icon("previous_branch",64,Gdk::Pixbuf::create_from_file(DATADIR"previous_branch.svg",64,64));
    Gtk::Image* black_player_im, *white_player_im;
    m_refBuilder->get_widget("image_black_player",black_player_im);
    m_refBuilder->get_widget("image_white_player",white_player_im);
    black_player_im->set_from_icon_name("player_black",Gtk::ICON_SIZE_LARGE_TOOLBAR);
    white_player_im->set_from_icon_name("player_white",Gtk::ICON_SIZE_LARGE_TOOLBAR);
    m_refBuilder->get_widget_derived("GobanWidget", m_gobanwidget);
    m_statusbar=Glib::RefPtr<Gtk::Statusbar>::cast_dynamic(m_refBuilder->get_object("statusbar_main"));
    m_gobanwidget->signal_stone_dropped().connect(sigc::mem_fun(this,&GobanWindow::on_stone_dropped));
    m_action_group=Glib::RefPtr<Gtk::ActionGroup>::cast_dynamic(m_refBuilder->get_object("actiongroup"));
    m_ui_manager=Glib::RefPtr<Gtk::UIManager>::cast_dynamic(m_refBuilder->get_object("uimanager"));
    //action_menu_file
    m_action_group->add(Gtk::Action::create("action_menu_file",_("_File")));
    //action_new
    m_action_group->add(Gtk::Action::create("action_new",Gtk::Stock::NEW),sigc::mem_fun(this,&GobanWindow::on_new_game));
    //action_new_gnugo
    m_action_group->add(Gtk::Action::create("action_new_gnugo",_("Play against the computer"), _("Start a new game against the AI gnugo")),
	                    sigc::mem_fun(this,&GobanWindow::on_play_against_gnugo));
    if(Glib::find_program_in_path("gnugo")==""){
        m_action_group->get_action("action_new_gnugo")->set_sensitive(false);
        m_action_group->get_action("action_new_gnugo")->set_tooltip(_("The program \"gnugo\" was not found in \"PATH\"."
                    "Make sure to install it and - if necessary - add its location to the \"PATH\" in oder to play against an AI."));
    }
    //action_close
    m_action_group->add(Gtk::Action::create("action_close",Gtk::Stock::CLOSE),sigc::mem_fun(this,&GobanWindow::on_close_game));
    //action_quit
    m_action_group->add(Gtk::Action::create("action_quit",Gtk::Stock::QUIT),sigc::mem_fun(this,&GobanWindow::on_quit));
    //action_save_as
    m_action_group->add(Gtk::Action::create("action_save_as",Gtk::Stock::SAVE_AS),sigc::mem_fun(this,&GobanWindow::on_save_as));
    //action_save
    m_action_group->add(Gtk::Action::create("action_save",Gtk::Stock::SAVE),sigc::mem_fun(this,&GobanWindow::on_save));
    //action_open
    m_action_group->add(Gtk::Action::create("action_open",Gtk::Stock::OPEN),sigc::mem_fun(this,&GobanWindow::on_open));
    //action_menu_edit
    m_action_group->add(Gtk::Action::create("action_menu_edit",_("Edit")));
    //action_game_properties
    m_action_group->add(Gtk::Action::create("action_game_properties",Gtk::Stock::PROPERTIES,_("Game _properties")),sigc::mem_fun(this,&GobanWindow::on_game_properties));
    //action_menu_view
    m_action_group->add(Gtk::Action::create("action_menu_view",_("_View")));
    //action_show_ticks
    m_action_group->add(Gtk::ToggleAction::create("action_show_ticks",_("Coordinates"),_("Show the coordinates on the goban"),false),sigc::mem_fun(this,&GobanWindow::on_show_ticks));
    m_action_group->add(Gtk::ToggleAction::create("action_show_comments",_("Comments"),_("Show the comments attached to the turns"),false),sigc::mem_fun(this,&GobanWindow::on_show_comments));
    m_action_group->add(Gtk::ToggleAction::create("action_show_next_nodes",_("Variations"),_("Show the available variations"),false),sigc::mem_fun(this,&GobanWindow::on_show_next_nodes));
    m_action_group->add(Gtk::ToggleAction::create("action_show_stats",_("Captures and time"),_("Show the number of captured stones and the time left")),sigc::mem_fun(this,&GobanWindow::on_show_stats));
    //action_menu_help
    m_action_group->add(Gtk::Action::create("action_menu_help",_("H_elp")));
    //action_about
    m_action_group->add(Gtk::Action::create("action_about",Gtk::Stock::ABOUT),sigc::mem_fun(this,&GobanWindow::on_about));
    //action_menu_go_to
    m_action_group->add(Gtk::Action::create("action_menu_go_to",_("_Navigation")));
    //action_next
    m_action_group->add(Gtk::Action::create("action_next",Gtk::Stock::GO_FORWARD,_("Next move"),_("Go to the next move")),sigc::mem_fun(this,&GobanWindow::on_next));
    //action_previous
    m_action_group->add(Gtk::Action::create("action_previous",Gtk::Stock::GO_BACK,_("Previous move"),_("Go to the previous move")),sigc::mem_fun(this,&GobanWindow::on_previous));
    m_action_group->add(Gtk::Action::create("action_beginning", Gtk::Stock::GOTO_FIRST,_("Previous embranchement"),_("Go to the previous variation embranchement")),sigc::mem_fun(this, &GobanWindow::on_go_beginning));
    m_action_group->add(Gtk::Action::create("action_last", Gtk::Stock::GOTO_LAST,_("Next embranchement"),_("Go to the next variation embranchement")),sigc::mem_fun(this, &GobanWindow::on_go_last));

    //action_next_branch
    m_action_group->add(Gtk::Action::create_with_icon_name("action_next_branch","next_branch",_("Next variation"),_("Go to the next variation")),sigc::mem_fun(this,&GobanWindow::on_next_branch));
    //action_previous_branch
    m_action_group->add(Gtk::Action::create_with_icon_name("action_previous_branch","previous_branch",_("Previous variation"),_("Go to the previous variation")),sigc::mem_fun(this,&GobanWindow::on_previous_branch));
    Gtk::RadioAction::Group mode_group;
    //action_mode_edit
    m_mode_edit = Gtk::RadioAction::create(mode_group,"action_mode_edit",Gtk::Stock::EDIT,_("Edit game"));
    m_action_group->add(m_mode_edit,sigc::mem_fun(this,&GobanWindow::on_mode_changed));
    //action_mode_play
    m_mode_play = Gtk::RadioAction::create(mode_group,"action_mode_play",Gtk::Stock::EXECUTE,_("Play game"));
    m_action_group->add(m_mode_play,sigc::mem_fun(this,&GobanWindow::on_mode_changed));
    //action_edit_group
    Gtk::RadioAction::Group edit_group;
    //action_edit_add_black
    m_edit_add_black = Gtk::RadioAction::create_with_icon_name(edit_group,"action_edit_add_black","add_black",_("Add black stones"),_("Add black stones without observing the go rules"));
    m_action_group->add(m_edit_add_black,sigc::mem_fun(this,&GobanWindow::on_edit_changed));
    //action_edit_add_white
    m_edit_add_white = Gtk::RadioAction::create_with_icon_name(edit_group,"action_edit_add_white","add_white",_("Add white stones"),_("Add white stones without observing the go rules"));
    m_action_group->add(m_edit_add_white,sigc::mem_fun(this,&GobanWindow::on_edit_changed));
    //action_edit_remove_stone
    m_edit_remove_stone = Gtk::RadioAction::create(edit_group,"action_edit_remove_stone",Gtk::Stock::REMOVE,_("Remove stones"));
    m_action_group->add(m_edit_remove_stone,sigc::mem_fun(this,&GobanWindow::on_edit_changed));
    //action_edit_play
    m_edit_play = Gtk::RadioAction::create_with_icon_name(edit_group,"action_edit_play","play",_("Play"),_("Make a move, adding it to the game tree."));
    m_action_group->add(m_edit_play,sigc::mem_fun(this,&GobanWindow::on_edit_changed));
    //action_menu_game
    m_action_group->add(Gtk::Action::create("action_menu_game",_("Game")));
    //action_resign
    m_action_group->add(Gtk::Action::create("action_resign", Gtk::Stock::NO, _("Resign")),sigc::mem_fun(this,&GobanWindow::on_resign));
    //action_pass
    m_action_group->add(Gtk::Action::create_with_icon_name("action_pass", "pass",_("Pass"),_("Pass one's turn")),sigc::mem_fun(this,&GobanWindow::on_pass));
    //comments changed
    m_comment_buffer=Glib::RefPtr<Gtk::TextBuffer>::cast_dynamic(m_refBuilder->get_object("textbuffer_comment"));
    m_comment_view=Glib::RefPtr<Gtk::TextView>::cast_dynamic(m_refBuilder->get_object("textview_comment"));
    m_comment_buffer->signal_modified_changed().connect(sigc::mem_fun(this,&GobanWindow::on_comment_changed));
    m_ui_manager->insert_action_group(m_action_group);
    add_accel_group(m_ui_manager->get_accel_group());
    ustring ui_layout=
        "<ui>"
        "   <menubar name='menubar'>"
        "       <menu action='action_menu_file'>"
        "           <menuitem action='action_new'/>"
		"			<menuitem action='action_new_gnugo'/>"
        "           <menuitem action='action_open'/>"
        "           <separator/>"
        "           <menuitem action='action_save'/>"
        "           <menuitem action='action_save_as'/>"
        "           <separator/>"
        "           <menuitem action='action_close'/>"
        "           <menuitem action='action_quit'/>"
        "       </menu>"
        "       <menu action='action_menu_edit'>"
        "           <menuitem action='action_mode_play'/>"
        "           <menuitem action='action_mode_edit'/>"
        "           <separator/>"
        "           <menuitem action='action_game_properties'/>"
        "       </menu>"
        "       <menu action='action_menu_view'>"
        "           <menuitem action='action_show_ticks'/>"
        "           <menuitem action='action_show_comments'/>"
        "           <menuitem action='action_show_next_nodes'/>"
        "           <menuitem action='action_show_stats'/>"
        "       </menu>"
        "       <menu action='action_menu_game'>"
        "           <menuitem action='action_pass'/>"
        "           <menuitem action='action_resign'/>"
        "           <separator/>"
        "           <menuitem action='action_edit_play'/>"
        "           <menuitem action='action_edit_add_black'/>"
        "           <menuitem action='action_edit_add_white'/>"
        "           <menuitem action='action_edit_remove_stone'/>"
        "       </menu>"
        "       <menu action='action_menu_go_to'>"
        "           <menuitem action='action_previous'/>"
        "           <menuitem action='action_next'/>"
        "           <menuitem action='action_beginning'/>"
        "           <menuitem action='action_last'/>"
        "           <menuitem action='action_previous_branch'/>"
        "           <menuitem action='action_next_branch'/>"
        "       </menu>"
        "       <menu action='action_menu_help'>"
        "           <menuitem action='action_about'/>"
        "       </menu>"
        "   </menubar>"
        "   <toolbar name='toolbar'>"
        "       <toolitem action='action_beginning'/>"
        "       <toolitem action='action_previous'/>"
        "       <toolitem action='action_next'/>"
        "       <toolitem action='action_last'/>"
        "       <toolitem action='action_previous_branch'/>"
        "       <toolitem action='action_next_branch'/>"
        "       <separator/>"
        "       <toolitem action='action_pass'/>"
        "       <toolitem action='action_resign'/>"
        "       <toolitem action='action_edit_play'/>"
        "       <toolitem action='action_edit_add_black'/>"
        "       <toolitem action='action_edit_add_white'/>"
        "       <toolitem action='action_edit_remove_stone'/>"
        "   </toolbar>"
        "   <popup name='popup'>"
        "       <menuitem action='action_edit_play'/>"
        "       <menuitem action='action_edit_add_black'/>"
        "       <menuitem action='action_edit_add_white'/>"
        "       <menuitem action='action_edit_remove_stone'/>"
        "       <separator/>"
        "       <menuitem action='action_pass'/>"
        "       <menuitem action='action_resign'/>"
        "   </popup>"
        "</ui>";
    m_ui_manager->add_ui_from_string(ui_layout);
    m_popup_menu=(Gtk::Menu*)(m_ui_manager->get_widget("/ui/popup"));
    Gtk::Widget* menubar=m_ui_manager->get_widget("/menubar");
    Gtk::Widget* toolbar=m_ui_manager->get_widget("/toolbar");
    Gtk::VBox* vbox_main=0;
    m_refBuilder->get_widget("vbox_main",vbox_main);
    vbox_main->pack_start(*menubar,Gtk::PACK_SHRINK);
    vbox_main->pack_start(*toolbar,Gtk::PACK_SHRINK);
    m_edit_play->set_active(true);
    actualize_ui();
    set_icon_from_file(DATADIR"icon.svg");
    m_refListStore = Gtk::ListStore::create(m_node_model);
    Gtk::TreeView* treeview=0;
    m_refBuilder->get_widget("NextNodesView",treeview);
    treeview->set_model(m_refListStore);
    treeview->append_column(_("id"),m_node_model.m_id);
    treeview->append_column(_("Comment"),m_node_model.m_text);
    treeview->set_tooltip_column(2);
    m_refselection=treeview->get_selection();
    m_refselection->signal_changed().connect(sigc::mem_fun(this,&GobanWindow::on_node_clicked));
    add_events(Gdk::BUTTON_PRESS_MASK);
    signal_button_press_event().connect
        (
         sigc::mem_fun(this,&GobanWindow::on_button_press_event)
        );
}

GobanWindow::~GobanWindow()
{
    if(m_goban_logic) delete m_goban_logic;
    if(m_opponent) delete m_opponent;
    GamePropertiesDialog* prop=0;
    AskPlayerDialog* aprop=0;
    OpenGameDialog* oprop=0;
    m_refBuilder->get_widget_derived("GamePropertiesDialog",prop);
    m_refBuilder->get_widget_derived("AskPlayerDialog",aprop);
    m_refBuilder->get_widget_derived("openfiledialog",oprop);
    delete prop;
    delete aprop;
    delete oprop;
}

void GobanWindow::on_show_ticks()
{
    Glib::RefPtr<Gtk::ToggleAction> ticks=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_ticks"));
    m_gobanwidget->SetShowTicks(ticks->get_active());
}

void GobanWindow::on_show_comments()
{
    Glib::RefPtr<Gtk::ToggleAction> comments=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_comments"));
    Glib::RefPtr<Gtk::Frame> frame_com=Glib::RefPtr<Gtk::Frame>::cast_dynamic(m_refBuilder->get_object("frame_comments"));
    Glib::RefPtr<Gtk::Frame> frame_nex=Glib::RefPtr<Gtk::Frame>::cast_dynamic(m_refBuilder->get_object("frame_next_moves"));
    Glib::RefPtr<Gtk::VPaned> vpan=Glib::RefPtr<Gtk::VPaned>::cast_dynamic(m_refBuilder->get_object("vbox_right"));
    frame_com->set_visible(comments->get_active());
    if(!comments->get_active() && !frame_nex->get_visible()){
        int pansize=vpan->get_allocation().get_width();
        int w,h;
        get_size(w,h);
        resize(w-pansize,h);
    }
}

void GobanWindow::on_show_next_nodes()
{
    Glib::RefPtr<Gtk::ToggleAction> nexmov=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_next_nodes"));
    Glib::RefPtr<Gtk::Frame> frame_com=Glib::RefPtr<Gtk::Frame>::cast_dynamic(m_refBuilder->get_object("frame_comments"));
    Glib::RefPtr<Gtk::Frame> frame_nex=Glib::RefPtr<Gtk::Frame>::cast_dynamic(m_refBuilder->get_object("frame_next_moves"));
    Glib::RefPtr<Gtk::VPaned> vpan=Glib::RefPtr<Gtk::VPaned>::cast_dynamic(m_refBuilder->get_object("vbox_right"));
    frame_nex->set_visible(nexmov->get_active());
    if(!nexmov->get_active() && !frame_com->get_visible()){
        int pansize=vpan->get_allocation().get_width();
        int w,h;
        get_size(w,h);
        resize(w-pansize,h);
    }
}

void GobanWindow::on_show_stats()
{
    Gtk::HBox *fg;
    Glib::RefPtr<Gtk::ToggleAction> st=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_stats"));
    m_refBuilder->get_widget("frame_game",fg);
    fg->set_visible(st->get_active());
    if(!st->get_active()){
        int frsz=fg->get_allocation().get_height();
        int w,h;
        get_size(w,h);
        resize(w,h-frsz);
    }
}

void GobanWindow::on_play_against_gnugo()
{
    actualize_node();
    if(m_goban_logic){
        if(!safe_to_delete_game())
            return;
        m_gobanwidget->SetBoard(0);
        delete m_goban_logic;
        if(m_opponent){
            delete m_opponent;
            m_opponent=0;
        }
        m_goban_logic=0;
        actualize_ui();
    }
    m_goban_logic=new GobanLogic();
    GamePropertiesDialog* prop=0;
    m_refBuilder->get_widget_derived("GamePropertiesDialog",prop);
    prop->AttachSgfNode(m_goban_logic->RootNode());
    prop->SetComputerGame();
    bool ok=false;
    while(!ok){
        int response=prop->run();
        if(response==Gtk::RESPONSE_OK){
            ustring pl_black=m_goban_logic->RootNode()->Property(SgfProperty::PB)->Value2U();
            ustring pl_white=m_goban_logic->RootNode()->Property(SgfProperty::PW)->Value2U();
            int size=ustring(_("Gnugo level")).size();
            if((pl_black.substr(0,size)!=_("Gnugo level")&&pl_white.substr(0,size)!=_("Gnugo level"))||
                    (pl_black.substr(0,size)==_("Gnugo level")&&pl_white.substr(0,size)==_("Gnugo level"))){
                Gtk::MessageDialog errordiag(_("Please select a single computer player."),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_OK,true);
                errordiag.run();
                errordiag.hide();
            } else {
                ok=true;
                m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->RootNode()));
                m_mode_play->set_active();
                if(pl_black.substr(0,size)==_("Gnugo level")){
                    int level=0;
                    istringstream istr(pl_black.erase(0,size+1));
                    istr>>level;
                    if(istr.fail() || level<0 || level> 10){
                        level=10;
                        Logger::log()<<"GobanWindow::on_play_against_gnugo: Failed to interpret wanted gnugo level. set it to 10 (max)"<<endl;
                    }
                    m_opponent=new GnugoOpponent(m_goban_logic->RootNode(),true,level,m_goban_logic->GetKomi());
                    m_human_black=false;
                    m_connection_played=m_opponent->signal_played().connect(sigc::mem_fun(this,&GobanWindow::on_opponent_played));
                    m_connection_final_state=m_opponent->signal_final_state().connect(sigc::mem_fun(this,&GobanWindow::on_final_state));
                    m_opponent->ToPlay(m_goban_logic->RootNode());
                    actualize_ui();
                } else {
                    int level=0;
                    istringstream istr(pl_white.erase(0,size+1));
                    istr>>level;
                    if(istr.fail() || level<0 || level> 10){
                        level=10;
                        Logger::log()<<"GobanWindow::on_play_against_gnugo: Failed to interpret wanted gnugo level. set it to 10 (max)"<<endl;
                    }
                    m_opponent=new GnugoOpponent(m_goban_logic->RootNode(),false,level,m_goban_logic->GetKomi());
                    m_connection_played=m_opponent->signal_played().connect(sigc::mem_fun(this,&GobanWindow::on_opponent_played));
                    m_connection_final_state=m_opponent->signal_final_state().connect(sigc::mem_fun(this,&GobanWindow::on_final_state));
                    m_human_black=true;
                }
                actualize_ui();
                m_goban_logic->SetModified(true);
            }
        }
    }
    prop->hide();
}

void GobanWindow::on_game_over()
{
    if(m_opponent){
        m_opponent->GetScore();
    }
    m_mode_edit->set_active();
    m_statusbar->push(_("Game over."));
    actualize_ui();
}

void GobanWindow::on_final_state(vector<Coord> dead, vector<Coord> black_terr, vector<Coord> white_terr)
{
    m_gobanwidget->SetBoard(m_goban_logic->SetFinalState(dead,black_terr,white_terr));
    actualize_ui();
}

bool GobanWindow::on_button_press_event(GdkEventButton* event)
{
    if(event->type==GDK_BUTTON_PRESS && event->button==3){
        int x,y;
        get_pointer(x,y);
        if(x>=m_gobanwidget->get_allocation().get_x() && x<m_gobanwidget->get_allocation().get_x()+m_gobanwidget->get_allocation().get_width()
                && y>=m_gobanwidget->get_allocation().get_y() && y<m_gobanwidget->get_allocation().get_y()+m_gobanwidget->get_allocation().get_height()){
            m_popup_menu->popup(event->button,event->time);
            return true;
        } else
            return false;
    } else
        return false;
}

void GobanWindow::on_resign()
{
    m_goban_logic->Resign();
    m_opponent->Resignation();
    if(m_connection_played.connected()) m_connection_played.disconnect();
    if(m_connection_final_state.connected()) m_connection_final_state.disconnect();
    m_mode_edit->set_active();
    m_statusbar->push(_("Game over."));
    actualize_ui();
}

void GobanWindow::on_pass()
{
    on_stone_dropped(Coord(ustring::compose("%1%2",
                    (char)((int)'a' + m_goban_logic->CurrentNode()->GetBoard().Size()),
                    (char)((int)'a' + m_goban_logic->CurrentNode()->GetBoard().Size()))));
    ustring plpassed;
    if(m_human_black) plpassed=_("Black passed");
    else plpassed=_("White passed");
    m_statusbar->push(plpassed);
    if(m_mode_play->get_active() && m_opponent && !m_opponent->IsPlaying() && !m_opponent->GameOver()){
        if(!m_goban_logic->CurrentNode()->GetFinal()) m_opponent->ToPlay(0);
        else on_game_over();
    }
}

void GobanWindow::on_node_clicked()
{
    Gtk::TreeModel::iterator iter=m_refselection->get_selected();
    if(iter){
        SgfNode* node=(*iter)[m_node_model.m_node];
        if(node->Property(SgfProperty::B)){
            if(node->Property(SgfProperty::B)->Value2C()==Coord(node->GetBoard().Size(),node->GetBoard().Size()))
                m_statusbar->push(_("Black passed"));
            else
                m_statusbar->push(ustring::compose(_("Black played %1"),humancoord(node->Property(SgfProperty::B)->Value2C())));
        } else if(node->Property(SgfProperty::W)){
            if(node->Property(SgfProperty::W)->Value2C()==Coord(node->GetBoard().Size(),node->GetBoard().Size()))
                m_statusbar->push(_("White passed"));
            else
                m_statusbar->push(ustring::compose(_("White played %1"),humancoord(node->Property(SgfProperty::W)->Value2C())));
        }
        m_gobanwidget->SetBoard(m_goban_logic->PlayNode((*iter)[m_node_model.m_node]));
    }
    actualize_ui();
}

void GobanWindow::on_edit_changed()
{
    actualize_node();
    actualize_ui();
}

void GobanWindow::on_opponent_played(SgfNode* node)
{
    if(node){
        m_gobanwidget->SetBoard(m_goban_logic->PlayNode(node));
        ustring pl;
        if(m_human_black){
            if(node->Property(SgfProperty::W)->Value2C()==Coord(node->GetBoard().Size(),node->GetBoard().Size()))
                pl="White passed";
            else
                pl=ustring::compose("White played %1",humancoord(node->Property(SgfProperty::W)->Value2C()));
        } else {
            if(node->Property(SgfProperty::B)->Value2C()==Coord(node->GetBoard().Size(),node->GetBoard().Size()))
                pl="Black passed";
            else
                pl=ustring::compose("Black played %1",humancoord(node->Property(SgfProperty::B)->Value2C()));
        }
        m_statusbar->push(pl);
    } else {
        Gtk::MessageDialog message(*this,_("This is the end of the game tree"),
                                   false,Gtk::MESSAGE_INFO,Gtk::BUTTONS_OK,true);
        message.run();
    }
    if(m_goban_logic->CurrentNode()->GetFinal()) on_game_over();
    actualize_ui();
}

void GobanWindow::on_mode_changed()
{
    actualize_node();
    if(m_mode_play->get_active()){
        if(!m_opponent) {
            m_opponent=new SgfRecordOpponent(m_goban_logic->RootNode());
            m_connection_played=m_opponent->signal_played().connect(sigc::mem_fun(this,&GobanWindow::on_opponent_played));
        }
    m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->NodeBeforeFirstMove()));
    m_human_black=m_goban_logic->Black();
    }
    actualize_ui();
}

void GobanWindow::on_stone_dropped(const Coord& coord)
{
    actualize_node();
    if(m_mode_play->get_active() && !m_opponent)
        throw GobanLogicError("[debug] GobanWindow::on_stone_dropped: play_mode on but no opponent allocated.");
    if(m_mode_play->get_active() && !m_opponent->IsPlaying() && !m_opponent->GameOver()){
        try{
            m_gobanwidget->SetBoard(m_goban_logic->PlayMove(coord));
            if(m_opponent && !m_goban_logic->CurrentNode()->GetFinal()) m_opponent->ToPlay(m_goban_logic->CurrentNode());
            ustring pl;
            if(m_goban_logic->CurrentNode()->Property(SgfProperty::B))
                pl=ustring::compose("Black played %1",humancoord(coord));
            else
                pl=ustring::compose("White played %1",humancoord(coord));
            m_statusbar->push(pl);
            actualize_ui();
        } catch(const GobanLogicError& err){
            if(err.code()==0) throw; //throw to upper level if the error is something different than an illegal move.
        }
    } else if(m_mode_edit->get_active()){
        if(m_edit_add_black->get_active()){
            m_problem=true;
            m_gobanwidget->SetBoard(m_goban_logic->AddStone(coord,true));
            m_statusbar->push(ustring::compose(_("Black stone added at %1"),humancoord(coord)));
        } else if(m_edit_add_white->get_active()){
            m_problem=true;
            m_gobanwidget->SetBoard(m_goban_logic->AddStone(coord,false));
            m_statusbar->push(ustring::compose(_("White stone added at %1"),humancoord(coord)));
        } else if(m_edit_remove_stone->get_active()){
            m_gobanwidget->SetBoard(m_goban_logic->RemoveStone(coord));
            m_statusbar->push(ustring::compose(_("Stone removed from %1"),humancoord(coord)));
        } else if(m_edit_play->get_active()){
            if(m_goban_logic->CurrentNode()==m_goban_logic->NodeBeforeFirstMove()){
                if(m_goban_logic->NodeBeforeFirstMove()->NChilds()==0 && m_problem){
                    AskPlayerDialog *dialog=0;
                    m_refBuilder->get_widget_derived("AskPlayerDialog",dialog);
                    int response=dialog->run();
                    if(response==Gtk::RESPONSE_OK){
                        m_goban_logic->SetBlack(dialog->Black());
                    }
                    dialog->hide();
                } else if(m_goban_logic->NodeBeforeFirstMove()->NChilds()!=0){
                    const SgfNode* first=m_goban_logic->NodeBeforeFirstMove()->ChildAt(0);
                    if(first->Property(SgfProperty::B)) m_goban_logic->SetBlack(true);
                    else if(first->Property(SgfProperty::B)) m_goban_logic->SetBlack(false);
                    else throw GobanLogicError("[debug] GobanWindow::on_stone_dropped: NodeBeforeFirstMove did not return the correct node!");
                } else {
                    m_goban_logic->SetBlack(true);
                }
            }
            try{
                m_gobanwidget->SetBoard(m_goban_logic->PlayMove(coord));
                if(m_goban_logic->CurrentNode()->Property(SgfProperty::B))
                    m_statusbar->push(ustring::compose(_("Black played %1"),humancoord(coord)));
                else
                    m_statusbar->push(ustring::compose(_("White played %1"),humancoord(coord)));
                actualize_ui();
            } catch(const GobanLogicError& err){
                if(err.code()==0) throw; //throw to upper level if the error is something different than an illegal move.
            }
        } 
    }
}

void GobanWindow::on_new_game()
{
    if(m_goban_logic){
        if(!safe_to_delete_game())
            return;
        m_gobanwidget->SetBoard(0);
        delete m_goban_logic;
        m_goban_logic=0;
        actualize_ui();
        m_problem=false;
    }
    m_goban_logic=new GobanLogic();
    GamePropertiesDialog* prop=0;
    m_refBuilder->get_widget_derived("GamePropertiesDialog",prop);
    prop->AttachSgfNode(m_goban_logic->RootNode());
    int response=prop->run();
    if(response==Gtk::RESPONSE_OK){
        m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->RootNode()));
        actualize_ui();
        m_goban_logic->SetModified(true);
    }
    m_statusbar->push(_("New game"));
    prop->hide();
}

void GobanWindow::on_game_properties()
{
    if(!m_goban_logic) return;
    GamePropertiesDialog* prop=0;
    m_refBuilder->get_widget_derived("GamePropertiesDialog",prop);
    prop->AttachSgfNode(m_goban_logic->RootNode());
    int response=prop->run();
    if(response==Gtk::RESPONSE_OK){
        m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->RootNode()));
        //TODO
        //needs to check whether the game properties did really change
        m_goban_logic->SetModified(true);
    }
    prop->hide();
}

void GobanWindow::on_open()
{
    actualize_node();
    if(m_goban_logic){
        if(!safe_to_delete_game())
            return;
        m_gobanwidget->SetBoard(0);
        delete m_goban_logic;
        m_goban_logic=0;
        actualize_ui();
        m_problem=false;
    }
    OpenGameDialog *opendialog;
    m_refBuilder->get_widget_derived("openfiledialog",opendialog);
    Gtk::FileFilter filter;
    filter.set_name(_("Game record (*.sgf)"));
    filter.add_pattern("*.sgf");
    opendialog->add_filter(filter);
    if(m_folder)
        opendialog->set_current_folder_file(m_folder);
    int response=opendialog->run();
    if(response==Gtk::RESPONSE_OK){
        m_file=opendialog->get_file();
        m_folder=opendialog->get_current_folder_file();
        Glib::RefPtr<Gio::DataInputStream> input=Gio::DataInputStream::create(m_file->read());
        string strin;
        ustring sgfstring("");
        while(input->read_line(strin)){
            sgfstring.append(strin.c_str());
            sgfstring.append("\n");
        }
        try{
            m_goban_logic=new GobanLogic(sgfstring);
            m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->NodeBeforeFirstMove()));
            if(opendialog->AsProblem()){
                if(m_opponent) {
                    if(m_connection_played.connected()) m_connection_played.disconnect();
                    if(m_connection_final_state.connected()) m_connection_final_state.disconnect();
                    delete m_opponent;
                    m_opponent=0;
                }
                m_opponent=new SgfRecordOpponent(m_goban_logic->RootNode());
                m_connection_played=m_opponent->signal_played().connect(sigc::mem_fun(this,&GobanWindow::on_opponent_played));
                m_mode_play->set_active();
                m_human_black=m_goban_logic->Black();
            }
            actualize_ui();
            m_statusbar->push(ustring::compose(_("File \"%1\" opened"),m_file->get_basename()));
        } catch(GobanLogicError err){
            Gtk::MessageDialog diag(ustring::compose(_("Error: could not open \"%1%2\""),m_file->get_path(),m_file->get_basename()),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_OK,true);
            diag.set_secondary_text(err.what());
            diag.run();
        } catch(SgfError err){
            Gtk::MessageDialog diag(ustring::compose(_("Error: could not open \"%1%2\""),m_file->get_path(),m_file->get_basename()),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_OK,true);
            diag.set_secondary_text(err.what());
            diag.run();
        } catch(Gio::Error err){
            Gtk::MessageDialog diag(ustring::compose(_("Error: could not open \"%1%2\""),m_file->get_path(),m_file->get_basename()),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_OK,true);
            diag.set_secondary_text(err.what());
            diag.run();
        }
    }
    opendialog->hide();
}

void GobanWindow::on_close_game()
{
    actualize_node();
    if(!safe_to_delete_game()){
        return;
    }
    m_gobanwidget->SetBoard(0);
    delete m_goban_logic;
    m_goban_logic=0;
    m_statusbar->push(_("Game closed"));
    actualize_ui();
    m_problem=false;
}

bool GobanWindow::safe_to_delete_game()
{
    if(m_goban_logic && m_goban_logic->Modified()){
        Gtk::MessageDialog save(*this,_("Save before leaving?"),false,Gtk::MESSAGE_WARNING,Gtk::BUTTONS_NONE,true);
        save.set_secondary_text(_("The modifications will be lost if you do not save them."));
        save.add_button(Gtk::Stock::YES,Gtk::RESPONSE_YES);
        save.add_button(Gtk::Stock::NO,Gtk::RESPONSE_NO);
        save.add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);
        int response=save.run();
        if(response==Gtk::RESPONSE_CANCEL) return false;
        else if(response==Gtk::RESPONSE_YES){
            on_save();
            if(m_goban_logic->Modified()) return false;
            else return true;
        } else {
            m_goban_logic->SetModified(false);
            return true;
        }
    } else {
        return true;
    }
}

void GobanWindow::on_quit()
{
    on_close_game();
    if(!m_goban_logic || !m_goban_logic->Modified())
        hide();
}

bool GobanWindow::on_delete_event(GdkEventAny* event)
{
    on_close_game();
    if(m_goban_logic && m_goban_logic->Modified())
        return true;
    else
        return false;
}

void GobanWindow::on_save_as()
{
    actualize_node();
    if(!m_goban_logic) return;
    Gtk::FileChooserDialog savedialog(*this,_("Save the game as"),Gtk::FILE_CHOOSER_ACTION_SAVE);
    savedialog.add_button(Gtk::Stock::SAVE_AS,Gtk::RESPONSE_OK);
    savedialog.add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);
    Gtk::FileFilter filter;
    filter.set_name(_("Game record (*.sgf)"));
    filter.add_pattern("*.sgf");
    savedialog.add_filter(filter);
    if(m_folder)
        savedialog.set_current_folder_file(m_folder);
    bool confirmed=false;
    while(!confirmed){
        int response=savedialog.run();
        if(response==Gtk::RESPONSE_OK){
            Glib::RefPtr<Gio::File> file=savedialog.get_file();
            if(file->query_exists()){
                Gtk::MessageDialog ask(ustring::compose(_("The file \"%1\" already exists. Do you want to overwrite it?"),file->get_basename()),false,Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO,true); //file already exsist
                ask.set_secondary_text(ustring::compose(_("The file \"%1\" already exists. Overwriting it will erase its contents."),file->get_path()));
                if(ask.run()==Gtk::RESPONSE_NO)
                    continue;
            }
            confirmed=true;
            m_file=file;
            m_folder=savedialog.get_current_folder_file();
            Glib::RefPtr<Gio::DataOutputStream> out;
            if(m_file->query_exists())
                out=Gio::DataOutputStream::create(m_file->replace());
            else
                out=Gio::DataOutputStream::create(m_file->create_file());
            out->put_string(m_goban_logic->RootNode()->str());
            m_statusbar->push(ustring::compose(_("Game saved under \"%1\""),m_file->get_basename()));
            m_goban_logic->SetModified(false);
            actualize_ui();
        } else
            confirmed=true;
    }
}

void GobanWindow::on_save()
{
    actualize_node();
    if(!m_goban_logic) return;
    if(!m_file)
        on_save_as();
    else{
        Gtk::MessageDialog ask(ustring::compose(_("Do you want to overwrite \"%1\"?"),m_file->get_path()),false,Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO,true); //file already exsist
        ask.set_secondary_text(_("This software is still in developpement. Overwriting a file created by an other program may result in permanent data loss."));
        if(ask.run()==Gtk::RESPONSE_YES){
            Glib::RefPtr<Gio::DataOutputStream> out=Gio::DataOutputStream::create(m_file->replace());
            out->put_string(m_goban_logic->RootNode()->str());
            m_statusbar->push(ustring::compose(_("Game saved under \"%1\""),m_file->get_basename()));
            m_goban_logic->SetModified(false);
        } else
            on_save_as();
    }
}

void GobanWindow::on_about()
{
    Gtk::AboutDialog about;
    vector<ustring> authors;
    authors.push_back("Bastien Dalla Piazza");
    about.set_authors(authors);
    about.set_program_name(PACKAGE_NAME);
    about.set_version(PACKAGE_VERSION);
    Glib::RefPtr<Gdk::Pixbuf> logo=Gdk::Pixbuf::create_from_file(DATADIR"logo.svg",-1,200);
    about.set_logo(logo);
    about.run();
}

void GobanWindow::on_next()
{
    actualize_node();
    m_gobanwidget->SetBoard(m_goban_logic->PlayNextNode());
    if(m_goban_logic->CurrentNode()->Property(SgfProperty::B))
        m_statusbar->push(ustring::compose(_("Black played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::B)->Value2C())));
    else if(m_goban_logic->CurrentNode()->Property(SgfProperty::W))
        m_statusbar->push(ustring::compose(_("White played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::W)->Value2C())));
    else
        m_statusbar->push(_("Next node"));
    actualize_ui();
}

void GobanWindow::on_previous()
{
    actualize_node();
    if(m_mode_play->get_active()){
        if(m_goban_logic->Black()!=m_human_black) m_gobanwidget->SetBoard(m_goban_logic->PlayPreviousNode());
        else{
            const Board* board = m_goban_logic->PlayPreviousNode();
            if(!m_goban_logic->HasPreviousNode()) m_gobanwidget->SetBoard(board);
            else m_gobanwidget->SetBoard(m_goban_logic->PlayPreviousNode());
        }
    } else
        m_gobanwidget->SetBoard(m_goban_logic->PlayPreviousNode());
    if(m_goban_logic->CurrentNode()->Property(SgfProperty::B))
        m_statusbar->push(ustring::compose(_("Black played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::B)->Value2C())));
    else if(m_goban_logic->CurrentNode()->Property(SgfProperty::W))
        m_statusbar->push(ustring::compose(_("White played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::W)->Value2C())));
    else
        m_statusbar->push(_("Previous node"));
    actualize_ui();
}

void GobanWindow::on_go_beginning()
{
    if(m_mode_edit->get_active()){
        m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->PreviousBranchingNode(m_goban_logic->CurrentNode())));
        m_statusbar->push(_("Beginning of the game"));
    }
    actualize_ui();
}

void GobanWindow::on_go_last()
{
    if(m_mode_edit->get_active()){
        m_gobanwidget->SetBoard(m_goban_logic->PlayNode(m_goban_logic->NextBranchingNode(m_goban_logic->CurrentNode())));
        m_statusbar->push(_("Last move"));
    }
    actualize_ui();
}

void GobanWindow::on_next_branch()
{
    actualize_node();
    m_gobanwidget->SetBoard(m_goban_logic->PlayNextBranch());
    if(m_goban_logic->CurrentNode()->Property(SgfProperty::B))
        m_statusbar->push(ustring::compose(_("Black played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::B)->Value2C())));
    else if(m_goban_logic->CurrentNode()->Property(SgfProperty::W))
        m_statusbar->push(ustring::compose(_("White played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::W)->Value2C())));
    else
        m_statusbar->push(_("Next branch"));
    actualize_ui();
}

void GobanWindow::on_previous_branch()
{
    actualize_node();
    m_gobanwidget->SetBoard(m_goban_logic->PlayPreviousBranch());
    if(m_goban_logic->CurrentNode()->Property(SgfProperty::B))
        m_statusbar->push(ustring::compose(_("Black played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::B)->Value2C())));
    else if(m_goban_logic->CurrentNode()->Property(SgfProperty::W))
        m_statusbar->push(ustring::compose(_("White played %1"),humancoord(m_goban_logic->CurrentNode()->Property(SgfProperty::W)->Value2C())));
    else
        m_statusbar->push(_("Previous branch"));
    actualize_ui();
}

void GobanWindow::on_comment_changed()
{
    if(m_goban_logic->CurrentNode()->Property(SgfProperty::C)){
        if(m_goban_logic->CurrentNode()->Property(SgfProperty::C)->Value2U()!=m_comment_buffer->get_text()){
            m_goban_logic->CurrentNode()->Property(SgfProperty::C)->SetValueAt(0,m_comment_buffer->get_text());
            m_goban_logic->SetModified(true);
        }
    } else if(m_comment_buffer->get_text().size()){
        m_goban_logic->CurrentNode()->AddProperty(SgfProperty::C,m_comment_buffer->get_text());
        m_goban_logic->SetModified(true);
    }
}

ustring GobanWindow::humancoord(Coord c)
{
    if(!m_goban_logic) return ustring("");
    if(c.i()>=8) return ustring::compose("%1%2",char(int('a')+c.i()+1),m_goban_logic->CurrentNode()->GetBoard().Size()-c.j()).uppercase();
    else return ustring::compose("%1%2",char(int('a')+c.i()),m_goban_logic->CurrentNode()->GetBoard().Size()-c.j()).uppercase();
}

void GobanWindow::actualize_node()
{
    if(!m_goban_logic) return;
    if(m_goban_logic->CurrentNode()->Property(SgfProperty::C)){
        if(m_goban_logic->CurrentNode()->Property(SgfProperty::C)->Value2U()!=m_comment_buffer->get_text()){
            m_goban_logic->CurrentNode()->Property(SgfProperty::C)->SetValueAt(0,m_comment_buffer->get_text());
            m_goban_logic->SetModified(true);
        }
    }else if(!m_comment_buffer->get_text().empty()){
        m_goban_logic->CurrentNode()->AddProperty(SgfProperty::C,m_comment_buffer->get_text());
        m_goban_logic->SetModified(true);
    }
}

void GobanWindow::actualize_ui()
{
    //TODO this method is a mess...
    if(m_goban_logic){
        Gtk::HBox *frame_game;
        m_refBuilder->get_widget("frame_game",frame_game);
        Glib::RefPtr<Gtk::ToggleAction> st=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_stats"));
        frame_game->set_visible(st->get_active());
        m_action_group->get_action("action_game_properties")->set_sensitive(true);
        Gtk::TreeView* node_view=0;
        m_refBuilder->get_widget("NextNodesView",node_view);
        node_view->set_sensitive(true);
        m_comment_view->set_sensitive(true);
        if(m_goban_logic->CurrentNode()->Property(SgfProperty::C)) {
            try{
                m_comment_buffer->set_text(m_goban_logic->CurrentNode()->Property(SgfProperty::C)->Value2U());
            } catch (const SgfPropertyError& err){
                Logger::log()<<"GobanWindow::actualize_ui: "<<err.what()<<endl;
                m_comment_buffer->set_text("");
            }
        } else {
            m_comment_buffer->set_text("");
        }
        m_refListStore->clear();
        for(int i=0; i<m_goban_logic->CurrentNode()->NChilds();++i){
            Gtk::TreeModel::iterator rowiter = m_refListStore->append();
            Gtk::TreeModel::Row row=*rowiter;
            row[m_node_model.m_id]=i+1;
            row[m_node_model.m_node]=m_goban_logic->CurrentNode()->ChildAt(i);
            if(m_goban_logic->CurrentNode()->ChildAt(i)->Property(SgfProperty::C)){
                ustring front="",comment=m_goban_logic->CurrentNode()->ChildAt(i)->Property(SgfProperty::C)->Value2U();
                row[m_node_model.m_full_text]=comment;
                while(front.size()==0 && comment.find("\n")!=ustring::npos){
                    front=comment.substr(0,comment.find("\n"));
                    comment=comment.substr(comment.find("\n"));
                    comment.erase(0,1);
                }
                if(front.size()!=0){
                    front.append("...");
                    row[m_node_model.m_text]=front;
                } else {
                    row[m_node_model.m_text]=comment;
                }
            }
            else
                row[m_node_model.m_text]="";
        }
        Gtk::Label *white_capt, *black_capt;
        m_refBuilder->get_widget("black_captures_number",black_capt);
        m_refBuilder->get_widget("white_captures_number",white_capt);
        black_capt->set_text(ustring::compose("%1",m_goban_logic->CurrentNode()->GetBoard().GetBlackCaptures()));
        white_capt->set_text(ustring::compose("%1",m_goban_logic->CurrentNode()->GetBoard().GetWhiteCaptures()));
        if(m_mode_play->get_active()){
            m_action_group->get_action("action_next")->set_sensitive(false);
            m_action_group->get_action("action_previous")->set_sensitive(m_goban_logic->CurrentNode()!=m_goban_logic->NodeBeforeFirstMove());
            m_action_group->get_action("action_beginning")->set_sensitive(false);
            m_action_group->get_action("action_last")->set_sensitive(false);
            m_action_group->get_action("action_next_branch")->set_sensitive(false);
            m_action_group->get_action("action_previous_branch")->set_sensitive(false);
            m_action_group->get_action("action_pass")->set_sensitive(true);
            m_action_group->get_action("action_resign")->set_sensitive(true);
            Glib::RefPtr<Gtk::ToggleAction> nexmov=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_next_nodes"));
            if(nexmov->get_active()){
                nexmov->set_active(false);
                nexmov->toggled();
                nexmov->set_active(true);
            }
            nexmov->set_sensitive(false);
            m_edit_add_black->set_sensitive(false);
            m_edit_add_white->set_sensitive(false);
            m_edit_play->set_sensitive(false);
            m_edit_remove_stone->set_sensitive(false);
            m_gobanwidget->SetBlack(m_goban_logic->Black());
            Gtk::Frame *next_move;
            m_refBuilder->get_widget("frame_next_moves",next_move);
            next_move->hide();
        } else {
            m_action_group->get_action("action_next")->set_sensitive(m_goban_logic->HasNextNode());
            m_action_group->get_action("action_previous")->set_sensitive(m_goban_logic->HasPreviousNode());
            m_action_group->get_action("action_next_branch")->set_sensitive(m_goban_logic->HasNextBranch());
            m_action_group->get_action("action_previous_branch")->set_sensitive(m_goban_logic->HasPreviousBranch());
            m_action_group->get_action("action_beginning")->set_sensitive(m_goban_logic->RootNode()!=m_goban_logic->CurrentNode());
            m_action_group->get_action("action_last")->set_sensitive(m_goban_logic->CurrentNode()->NChilds());
            m_action_group->get_action("action_pass")->set_sensitive(true);
            m_action_group->get_action("action_resign")->set_sensitive(true);
            m_edit_add_black->set_sensitive(true);
            m_edit_add_white->set_sensitive(true);
            m_edit_remove_stone->set_sensitive(true);
            m_edit_play->set_sensitive(true);
            Glib::RefPtr<Gtk::ToggleAction> nexmov=Glib::RefPtr<Gtk::ToggleAction>::cast_dynamic(m_action_group->get_action("action_show_next_nodes"));
            nexmov->set_sensitive(true);
            nexmov->toggled();
            if(m_edit_add_black->get_active()) m_gobanwidget->SetBlack(true);
            else if(m_edit_add_white->get_active()) m_gobanwidget->SetBlack(false);
            else if(m_edit_remove_stone->get_active()) m_gobanwidget->SetVoidCursor();
            else if(m_edit_play->get_active()){
                if(m_goban_logic->CurrentNode()==m_goban_logic->NodeBeforeFirstMove() && m_goban_logic->NodeBeforeFirstMove()->NChilds()==0 && m_problem)
                    m_gobanwidget->SetVoidCursor();
                else
                    m_gobanwidget->SetBlack(m_goban_logic->Black());
            }
        }
        m_action_group->get_action("action_mode_edit")->set_sensitive(true);
        m_action_group->get_action("action_mode_play")->set_sensitive(true);
        if(m_file){
            ustring filename,path=m_file->get_path();
            filename=path.substr(path.rfind("/")+1);
            path=path.substr(0,path.rfind("/"));
            set_title(ustring::compose("%1 - ggoban",filename,path));
        } else {
            set_title(ustring::compose("%1: %2 vs %3: %4 - ggoban",m_goban_logic->RootNode()->Property(SgfProperty::PB)->Value2U(),
                    m_goban_logic->CurrentNode()->GetBoard().GetBlackCaptures(),
                    m_goban_logic->RootNode()->Property(SgfProperty::PW)->Value2U(),
                    m_goban_logic->CurrentNode()->GetBoard().GetWhiteCaptures()+m_goban_logic->GetKomi()));
        }
        Gtk::Frame *frame_score;
        m_refBuilder->get_widget("frame_score",frame_score);
        if(m_goban_logic->CurrentNode()->GetFinal()){
            Gtk::Label* score;
            m_refBuilder->get_widget("label_score",score);
            ustring strscore=m_goban_logic->GetScore();
            cout<<strscore<<endl;
            if(strscore=="Draw" || strscore=="0"){
                score->set_markup(_("Jigo"));
            } else if(strscore=="W+R" || strscore=="W+Resign")
                score->set_markup(_("<big><b>White wins by black resignation</b></big>"));
            else if(strscore=="B+R" || strscore== "B+Resign")
                score->set_markup(_("<big><b>Black wins by white resignation</b></big>"));
            else if(strscore=="W+Forfeit" || strscore=="W+F")
                score->set_markup(_("<big><b>White wins by black forfeit</b></big>"));
            else if(strscore=="B+Forfeit" || strscore=="B+F")
                score->set_markup(_("<big><b>Black wins by white forfeit</b></big>"));
            else if(strscore=="W+Time" || strscore=="W+T")
                score->set_markup(_("<big><b>White wins by black overtime</b></big>"));
            else if(strscore=="B+Time" || strscore=="B+T")
                score->set_markup(_("<big><b>Black wins by white overtime</b></big>"));
            else if(strscore=="Void")
                score->set_markup(_("<big><b>No result</b></big>"));
            else if(strscore=="?")
                score->set_markup(_("<big><b>Result unknown</b></big>"));
            else {
                istringstream strin(strscore.substr(2));
                double moku=0;
                strin>>moku;
                if(strscore[0]=='W') score->set_markup(ustring::compose(_("<big><b>White wins by %1 moku</b></big>"),moku));
                else score->set_markup(ustring::compose(_("<big><b>Black wins by %1 moku</b></big>"),moku));
            }
            if(strscore!="?") frame_score->show();
            Gtk::HBox *hbox_score;
            m_refBuilder->get_widget("hbox_score",hbox_score);
            if(m_goban_logic->CurrentNode()->Property(SgfProperty::TB) && m_goban_logic->CurrentNode()->Property(SgfProperty::TW)){
                double black_score=m_goban_logic->CurrentNode()->GetBoard().GetBlackCaptures();
                double white_score=m_goban_logic->CurrentNode()->GetBoard().GetWhiteCaptures()+m_goban_logic->GetKomi();
                black_score+=m_goban_logic->CurrentNode()->Property(SgfProperty::TB)->NContent();
                white_score+=m_goban_logic->CurrentNode()->Property(SgfProperty::TW)->NContent();
                Gtk::Label *label_white_score_num, *label_black_score_num;
                m_refBuilder->get_widget("label_white_score_num",label_white_score_num);
                m_refBuilder->get_widget("label_black_score_num",label_black_score_num);
                label_black_score_num->set_text(ustring::compose(_("%1 moku"),black_score));
                label_white_score_num->set_text(ustring::compose(_("%1 moku"),white_score));
                hbox_score->show();
            } else{
                hbox_score->hide();
                int frsz=hbox_score->get_allocation().get_height();
                int w,h;
                get_size(w,h);
                resize(w,h-frsz);
            }

        } else {
            frame_score->hide();
            int frsz=frame_score->get_allocation().get_height();
            int w,h;
            get_size(w,h);
            resize(w,h-frsz);
        }
    } else {
        m_comment_view->set_sensitive(false);
        Gtk::HBox *frame_game;
        m_refBuilder->get_widget("frame_game",frame_game);
        frame_game->hide();
        Gtk::TreeView* node_view=0;
        m_refBuilder->get_widget("NextNodesView",node_view);
        node_view->set_sensitive(false);
        m_action_group->get_action("action_next")->set_sensitive(false);
        m_action_group->get_action("action_previous")->set_sensitive(false);
        m_action_group->get_action("action_next_branch")->set_sensitive(false);
        m_action_group->get_action("action_beginning")->set_sensitive(false);
        m_action_group->get_action("action_last")->set_sensitive(false);
        m_action_group->get_action("action_previous_branch")->set_sensitive(false);
        m_action_group->get_action("action_game_properties")->set_sensitive(false);
        m_action_group->get_action("action_mode_edit")->set_sensitive(false);
        m_action_group->get_action("action_mode_play")->set_sensitive(false);
        m_action_group->get_action("action_pass")->set_sensitive(false);
        m_action_group->get_action("action_resign")->set_sensitive(false);
        m_edit_play->set_sensitive(false);
        m_edit_add_black->set_sensitive(false);
        m_edit_add_white->set_sensitive(false);
        m_edit_remove_stone->set_sensitive(false);
        set_title("ggoban");
    }
}

