#include "SgfNode.h"

SgfNode::~SgfNode()
{
    while(m_childs.size()>0)
        delete m_childs.back();
    if(m_parent) m_parent->RemoveChild(this);
    list<SgfProperty*>::iterator rm=m_prop.begin();
    while(rm!=m_prop.end()){
        delete *rm;
        rm++;
    }
}

void SgfNode::RemoveChild(SgfNode* child)
{
    vector<SgfNode*>::iterator rm=find(m_childs.begin(),m_childs.end(),child);
    if(rm!=m_childs.end())
        m_childs.erase(rm);
    else
        cerr<<"SgfNode::RemoveChild: SgfNode* child not founf at: "<<child<<endl;
}

void SgfNode::RemoveProperty(SgfProperty* prop)
{
    list<SgfProperty*>::iterator rm=find(m_prop.begin(),m_prop.end(),prop);
    if(rm!=m_prop.end())
        m_prop.erase(rm);
    else
        cerr<<"SgfNode::RemoveProp: SgfProperty prop not found at: "<<prop<<endl;
}

bool SgfNode::HasNextBranch(const SgfNode* child) const
{
    vector<SgfNode*>::const_iterator ch=find(m_childs.begin(),m_childs.end(),child);
    ch++;
    if(ch==m_childs.end()) return false;
    else return true;
}
        
const SgfNode* SgfNode::NextBranch(const SgfNode* child) const
{
    vector<SgfNode*>::const_iterator ch=find(m_childs.begin(),m_childs.end(),child);
    ch++;
    if(ch==m_childs.end())
        throw SgfError("No next branch.",this);
    else
        return *ch;
}

SgfNode* SgfNode::NextBranch(const SgfNode* child)
{
    vector<SgfNode*>::iterator ch=find(m_childs.begin(),m_childs.end(),child);
    ch++;
    if(ch==m_childs.end())
        throw SgfError("SgfNode::NextBranch: No next branch.",this);
    else
        return *ch;
}

bool SgfNode::HasPreviousBranch(const SgfNode* child) const
{
    vector<SgfNode*>::const_iterator ch=find(m_childs.begin(),m_childs.end(),child);
    if(ch==m_childs.begin()) return false;
    else return true;
}

const SgfNode* SgfNode::PreviousBranch(const SgfNode* child) const
{
    vector<SgfNode*>::const_iterator ch=find(m_childs.begin(),m_childs.end(),child);
    if(ch==m_childs.begin())
        throw SgfError("SgfNode::PreviousBranch: No previous branch.",this);
    else {
        ch--;
        return *ch;
    }
}

SgfNode* SgfNode::PreviousBranch(const SgfNode* child)
{
    vector<SgfNode*>::iterator ch=find(m_childs.begin(),m_childs.end(),child);
    if(ch==m_childs.begin())
        throw SgfError("SgfNode::PreviousBranch: No previous branch.",this);
    else {
        ch--;
        return *ch;
    }
}


const SgfProperty* SgfNode::Property(const SgfProperty::SgfPropEnum& ident) const
{
    const SgfProperty* out=0;
    list<SgfProperty*>::const_iterator iter=m_prop.begin();
    while(iter!=m_prop.end()){
        if((*iter)->PropIdent()==ident)
            out=*iter;
        iter++;
    }
    return out;
}

SgfProperty* SgfNode::Property(const SgfProperty::SgfPropEnum& ident)
{
    SgfProperty* out=0;
    list<SgfProperty*>::const_iterator iter=m_prop.begin();
    while(iter!=m_prop.end()){
        if((*iter)->PropIdent()==ident)
            out=*iter;
        iter++;
    }
    return out;
}

ostream& operator<<(ostream& out,const SgfNode& node)
{
    if(node.m_parent==0 || node.m_parent->NChilds()>1)
        out<<"\n(";
    out<<"\n;";
    list<SgfProperty*>::const_iterator iter=node.m_prop.begin();
    while(iter!=node.m_prop.end()){
        out<<*(*iter);
        iter++;
    }
    for(int i=0;i<node.m_childs.size();++i){
        out<<*node.m_childs[i];
    }
    if(node.m_parent==0 || node.m_parent->NChilds()>1)
        out<<")\n";
    return out;
}

string SgfNode::str()
{
    ostringstream out;
    out<<*this;
    return out.str();
}

ostream& operator<<(ostream& out,const SgfError& err)
{
    out<<err.what()<<" at node: "<<err.error_node()<<endl;
    return out;
}

SgfNode* SgfNode::BuildTree(const ustring& in)
{
    stack<SgfNode*> branching_nodes;
    SgfProperty *current_prop=0;
    SgfNode *current=0;
    SgfNode* root=0;
    bool push_last=false, in_prop=false, in_node=false;
    ustring node_content="";
    uchar c;
    Glib::ustring::const_iterator iter=in.begin(),prev_iter;
    int inc=0;
    while(iter!=in.end()){
        c=*iter;
        prev_iter=iter;
        prev_iter--;
        switch(c){
            case '(':
                {
                    if(!in_prop)
                        push_last=true;
                    else
                        node_content.push_back(c);
                }
                break;
            case ')':
                {
                    if(!in_prop){
                        if(current && node_content.size()){
                            //current->SetContent(node_content);
                            node_content="";
                        }
                        if(current && in_node) current->SetFinal(true);
                        in_node=false;
                        current=branching_nodes.top();
                        branching_nodes.pop();
                    } else
                        node_content.push_back(c);
                }
                break;
            case ';':
                {
                    if(!in_prop){
                        if(current && node_content.size()){
                            //current->SetContent(node_content);
                            node_content="";
                        }
                        if(push_last){
                            branching_nodes.push(current);
                            push_last=false;
                        }
                        if(!current && root){
                            throw SgfError("root node is defined more than once.",root);
                        } else if(!current){
                            root=new SgfNode(0);
                            current=root;
                        } else {
                            current=new SgfNode(current);
                        }
                        in_node=true;
                    } else
                        node_content.push_back(c);
                }
                break;
            case '[':
                {
                    if(inc==0)
                        throw SgfError("sgf file must start with a new branch, i.e. a '('.",current);
                    else if(*prev_iter!='\\' && in_prop)
                        throw SgfError("non escaped '[' while inside of property definition '[...]'.",current);
                    else if(*prev_iter!='\\'){
                        in_prop=true;
                        if(node_content.size() && current){
                            current_prop=new SgfProperty();
                            current_prop->SetPropertyIdent(node_content);
                            current->AddProperty(current_prop);
                            node_content="";
                        } else if(!current)
                            throw SgfError("Encoutered '[' without a node defined.",current);
                    } else
                        node_content.push_back(c);
                }
                break;
            case ']':
                {
                    if(inc==0)
                        throw SgfError("sgf file must start with a new branch, i.e. a '('.",current);
                    else if(*prev_iter!='\\' && !in_prop)
                        throw SgfError("non escaped ']' while outside of property definition '[...]'.",current);
                    else if(*prev_iter!='\\'){
                        in_prop=false;
                        if(!current_prop)
                            throw SgfError("Encoutered ']' without property defined.",current);
                        else if(node_content.size()){
                            current_prop->AddValue(node_content,true);
                            node_content="";
                        }
                    } else
                        node_content.push_back(c);
                }
                break;
            case '\n':
                {
                    if(in_prop){
                        node_content.push_back(c);
                    }
                }
                break;
            case '\r':
                {
                    if(in_prop){
                        node_content.push_back(c);
                    }
                }
                break;
            case '\t':
                {
                    if(in_prop)
                        node_content.push_back(c);
                }
                break;
            case ' ':
                {
                    if(in_prop)
                        node_content.push_back(c);
                }
                break;
            default:
                node_content.push_back(c);
                break;
        }
        iter++;
        inc++;
    }
    if(branching_nodes.size())
        throw SgfError("unpaired '()' detected.",root);
    else if(in_prop)
        throw SgfError("unpaired '[]' detected.",root);
    else if(push_last)
        throw SgfError("empty branch detected.",root);
    else{
        return root;
    }
}

string SgfNode::get_sgf_charset(const string& in)
{
    int pos1,pos2;
    pos1=in.find("CA[");
    if(pos1==string::npos)
        return "ISO-8859-1"; //default
    else{
        pos2=in.find("]",pos1);
        if(pos2==string::npos){
            cerr<<"SgfNode::get_sgf_charset: Bad formatting: unpaired \"[]\""<<endl;
            return "";
        }else{
            return in.substr(pos1,pos2-pos1-1);
        }
    }
}
