#include "SgfProperty.h"

SgfProperty::SgfProperty(SgfProperty::SgfPropEnum id, ustring value)
    :m_prop_ident(id), m_composed(false)
{
    if(m_map_str2e.size()==0) BuildPropDict();
    if(id>=0 && m_map_e2str.find((SgfPropEnum)id)==m_map_e2str.end()){
        ostringstream err;
        err<<"SgfProperty::SgfProperty(const SgfPropEnum&): unknown id \""<<id<<"\".";
        throw SgfPropertyError(err.str());
    } else if(id>=0) {
        m_prop_str_ident=m_map_e2str[(SgfPropEnum)id];
        m_composed=might_be_composed(id);
        if(value.size()>0)
            AddValue(value);
    }
}

bool SgfProperty::might_be_composed(SgfProperty::SgfPropEnum id)
{
    //lists properties that might be composed
    //(semicolon in propValue must be interpreted as a syntaxic asset).
    //only lists properties currently supported by the parser
    if(id==LB) return true;
    else return false;
}

void SgfProperty::SetPropertyIdent(const ustring& ident)
{
    if(m_map_str2e.find(ident)!=m_map_str2e.end()){
        m_prop_ident=m_map_str2e[ident];
        m_composed=might_be_composed((SgfPropEnum)m_prop_ident);
    } else
        m_prop_ident=-1;
    m_prop_str_ident=ident;
}

void SgfProperty::SetPropertyIdent(const SgfPropEnum& ident)
{
    if(m_map_e2str.find(ident)!=m_map_e2str.end()){
        m_prop_str_ident=m_map_e2str[ident];
        m_prop_ident=ident;
        m_composed=might_be_composed(ident);
    } else {
        throw SgfPropertyError("SgfProperty::SetPropertyIdent: Unknown identificator");
    }
}

void SgfProperty::SetValueAt(int n, ustring val,bool form)
{
    list<ustring>::iterator iter=prop_value_at(n);
    if(is_formatted_text(m_prop_ident) && !form){
        ustring esc="[]\\";
        if(m_composed) esc.push_back(':');
        *iter=add_escape_char(val,esc);
    } else
        *iter=val;
}

void SgfProperty::AddValue(const ustring& content,bool form)
{
    if(!is_formatted_text(m_prop_ident) || form){
        m_prop_value.push_back(content);
    } else {
        ustring esc="[]\\";
        if(m_composed) esc.push_back(':');
        m_prop_value.push_back(add_escape_char(content,esc));
    }
}

void SgfProperty::RemoveValue(const ustring& value)
{
    list<ustring>::iterator iter = m_prop_value.begin();
    while(iter!=m_prop_value.end() && *iter!=value) iter++;
    if(iter==m_prop_value.end()) throw SgfPropertyError("SgfProperty::RemoveValue: No matching value to remove.");
    m_prop_value.erase(iter);
}

bool SgfProperty::HasValue(const ustring& value) const
{
    list<ustring>::const_iterator iter = m_prop_value.begin();
    while(iter!=m_prop_value.end() && *iter!=value) iter++;
    return iter!=m_prop_value.end();
}

ostream& operator<<(ostream& out, const SgfProperty & sgfprop)
{
    out<<sgfprop.m_prop_str_ident.raw();
    list<ustring>::const_iterator iter = sgfprop.m_prop_value.begin();
    while(iter != sgfprop.m_prop_value.end()){
        out<<'['<<(*iter).raw()<<']';
        iter++;
    }
    return out;
}

list<ustring>::const_iterator SgfProperty::prop_value_at(const int& i) const
{
    if(i==0) return m_prop_value.begin();
    else if(i<0 || i>=m_prop_value.size())
        throw SgfPropertyError("SgfProperty::prop_value_at(const int&): invalid index.");
    else{
        list<ustring>::const_iterator iter=m_prop_value.begin();
        int ai=0;
        while(ai!=i){
            iter++;
            ai++;
        }
        return iter;
    }
}

list<ustring>::iterator SgfProperty::prop_value_at(const int& i)
{
    if(i==0) return m_prop_value.begin();
    else if(i<0 || i>=m_prop_value.size())
        throw SgfPropertyError("SgfProperty::prop_value_at(const int&): invalid index.");
    else{
        list<ustring>::iterator iter=m_prop_value.begin();
        int ai=0;
        while(ai!=i){
            iter++;
            ai++;
        }
        return iter;
    }
}

ustring SgfProperty::Value2U(list<ustring>::const_iterator iter, bool composed) const
{
    if(iter==m_prop_value.end()) throw SgfPropertyError("SgfProperty::Value2U: no value is defined for the property.");
    if(m_composed){
        ustring first,second;
        split_composed(*iter,first,second);
        if(!composed) return remove_escape_char(first);
        else return remove_escape_char(second);
    } else return remove_escape_char(*iter);
}

int SgfProperty::Value2I(list<ustring>::const_iterator iter, bool composed) const
{
    if(iter==m_prop_value.end()) throw SgfPropertyError("SgfProperty::Value2I: no value is defined for the property.");
    ustring c;
    if(m_composed){
        ustring first,second;
        split_composed(*iter,first,second);
        if(!composed) c=first;
        else c=second;
    } else c=*iter;
    if(!c.is_ascii()) throw SgfPropertyError("SgfProperty::Value2I: invalid characters.");
    else {
        istringstream in(c.raw());
        int out;
        in>>out;
        if(in. fail() || in.bad())
            throw SgfPropertyError("SgfProperty::Value2I: invalid characters.");
        else
            return out;
    }
}

double SgfProperty::Value2D(list<ustring>::const_iterator iter, bool composed) const
{
    if(iter==m_prop_value.end()) throw SgfPropertyError("SgfProperty::Value2D: no value is defined for the property.");
    ustring c;
    if(m_composed){
        ustring first,second;
        split_composed(*iter,first,second);
        if(!composed) c=first;
        else c=second;
    } else c=*iter;
    if(!c.is_ascii()) throw SgfPropertyError("SgfProperty::Value2D: invalid characters.");
    else {
        istringstream in(c.raw());
        double out;
        in>>out;
        if(in. fail() || in.bad())
            throw SgfPropertyError("SgfProperty::Value2D: invalid characters.");
        else
            return out;
    }
}

Coord SgfProperty::Value2C(list<ustring>::const_iterator iter, bool composed) const
{
    if(iter==m_prop_value.end()) throw SgfPropertyError("SgfProperty::Value2C: no value is defined for the property.");
    ustring c;
    if(m_composed){
        ustring first,second;
        split_composed(*iter,first,second);
        if(!composed) c=first;
        else c=second;
    } else c=*iter;
    try{
        Coord out(c);
        return out;
    } catch(CoordError& err) {
        throw SgfPropertyError(err.what().append(" | encountered in SgfProperty::Value2C"));
    }
}

ustring SgfProperty::add_escape_char(ustring in, ustring chars)
{
    ustring out="";
    for(int n=0;n<in.size();++n){
        for(int c=0;c<chars.size();++c){
            if(in[n]==chars[c])
                out.push_back('\\');
        }
        out.push_back(in[n]);
    }
    return out;
}

ustring SgfProperty::remove_escape_char(ustring in)
{
    ustring out="";
    for(int n=0;n<in.size();++n){
        if(in[n]=='\\'){
            if(in[n+1]=='\n'){
                ++n;
            }
        } else if(in[n]=='\t')
            out.push_back(' ');
        else {
            out.push_back(in[n]);
        }
    }
    return out;
}

bool SgfProperty::is_formatted_text(int id)
{
    return (id==SgfProperty::C || id==SgfProperty::GC);
}

void SgfProperty::split_composed(const ustring& in, ustring& out1, ustring& out2) const
{
    bool found=false;
    int index=0;
    while(!found && index!=in.size()){
        index=in.find(":",index);
        if(index==ustring::npos) found=true;
        else if(index==0 || index==in.size()-1) throw SgfPropertyError("SgfProperty::split_composed: formatting error, a property value should not start or end with a (unescaped) ':'.");
        else if(in[index-1]!='\\') found=true;
        else ++index;
    }

    if(index==ustring::npos){
        out1=in;
        out2="";
    } else {
        out1=in.substr(0,index);
        out2=in.substr(index+1);
    }
}

ostream& operator<<(ostream& os, const SgfPropertyError& err)
{
    os<<"SgfPropertyError: "<<err.what();
    return os;
}
